"""Test for places models."""

from tomtom_apis.places.models import Connector, Context


def test_connector_post_deserialize() -> None:
    """Test the Connector __post_deserialize__ method."""
    connector1 = {"id": "1", "currentA": 32, "currentType": "AC3", "ratedPowerKW": 22.0, "type": "IEC62196Type2Outlet", "voltageV": 400}
    connector2 = {"id": "1", "currentA": 32, "currentType": "AC3", "ratedPowerKW": 22.0, "connectorType": "IEC62196Type2Outlet", "voltageV": 400}

    assert Connector.from_dict(connector1) == Connector.from_dict(connector2)


def test_context_pre_deserialize() -> None:
    """Test the Context __pre_deserialize__ method."""
    context1 = Context.from_dict({"inputQuery": "foo", "geoBias": {"position": {}}})
    context2 = Context.from_dict({"inputQuery": "foo"})

    assert context1 == context2



"""EV Search test."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.places import EVSearchApi
from tomtom_apis.places.models import EVChargingStationsAvailabilityResponse, EvSearchByIdParams, EvSearchNearbyParams, SearchResponse


@pytest.fixture(name="ev_search_api")
async def fixture_ev_search_api() -> AsyncGenerator[EVSearchApi]:
    """Fixture for EVSearchApi."""
    options = ApiOptions(api_key=API_KEY)
    async with EVSearchApi(options) as ev_search:
        yield ev_search


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/ev_search/get_ev_search_nearby.json"], indirect=True)
async def test_deserialization_get_ev_search_nearby(ev_search_api: EVSearchApi) -> None:
    """Test the get_ev_search_nearby method."""
    response = await ev_search_api.get_ev_search_nearby(
        params=EvSearchNearbyParams(
            lat=52.364941,
            lon=4.8935986,
            radius=10,
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)

    # Test that the fields Connector.type and Connector.connectorType are set the same after deserialization.
    assert response.results[0].chargingStations
    first_connector = response.results[0].chargingStations[0].chargingPoints[0].connectors[0]
    assert first_connector.type_ == first_connector.connectorType


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/ev_search/get_ev_search_by_id.json"], indirect=True)
async def test_deserialization_get_ev_search_by_id(ev_search_api: EVSearchApi) -> None:
    """Test the get_ev_search_by_id method."""
    response = await ev_search_api.get_ev_search_by_id(params=EvSearchByIdParams(id="RS*ORI*E1161"))

    assert response
    assert isinstance(response, SearchResponse)

    # Test that the fields Connector.type and Connector.connectorType are set the same after deserialization.
    assert response.results[0].chargingStations
    first_connector = response.results[0].chargingStations[0].chargingPoints[0].connectors[0]
    assert first_connector.type_ == first_connector.connectorType


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/ev_search/get_ev_charging_stations_availability.json"], indirect=True)
async def test_get_ev_charging_stations_availability(ev_search_api: EVSearchApi) -> None:
    """Test the get_ev_charging_stations_availability method."""
    response = await ev_search_api.get_ev_charging_stations_availability(chargingAvailability="75502858-a491-36fe-7128-23d400153b86")

    assert response
    assert isinstance(response, EVChargingStationsAvailabilityResponse)



"""Examples of search API calls."""

# pylint: disable=duplicate-code

import asyncio
import os

from tomtom_apis import ApiOptions
from tomtom_apis.models import ViewType
from tomtom_apis.places import SearchApi
from tomtom_apis.places.models import Geometry, GeometrySearchParams, GeometrySearchPostData, NearbySearchParams, PlaceByIdParams, RelatedPoisType


async def get_place_by_id(api_key: str) -> None:
    """Example for get_place_by_id."""
    search_id = "528009004256119"
    async with SearchApi(ApiOptions(api_key=api_key)) as search_api:
        response = await search_api.get_place_by_id(params=PlaceByIdParams(entityId=search_id))

        if response.results and response.results[0].poi is not None:
            print(f"\nPlace by id: '{search_id}' = {response.results[0].poi.name}")
        else:
            print(f"No POI data found for place id: {search_id}")


async def get_nearby_search(api_key: str) -> None:
    """Example for get_nearby_search."""
    async with SearchApi(ApiOptions(api_key=api_key)) as search_api:
        radius = 10000
        response = await search_api.get_nearby_search(
            lat=29.7604,
            lon=-95.3698,
            params=NearbySearchParams(
                radius=radius,
                brandSet=["McDonald's"],
            ),
        )

        print(f"\nThere are {response.summary.totalResults} McDonald's restaurants in a {radius} meter radius of the centre of Houston")


async def post_geometry_search(api_key: str) -> None:
    """Example for post_geometry_search."""
    async with SearchApi(ApiOptions(api_key=api_key)) as search_api:
        response = await search_api.post_geometry_search(
            query="pizza",
            params=GeometrySearchParams(
                categorySet=["7315"],
                view=ViewType.UNIFIED,
                relatedPois=RelatedPoisType.OFF,
            ),
            data=GeometrySearchPostData(
                geometryList=[
                    Geometry(
                        type="POLYGON",
                        vertices=[
                            "37.7524152343544, -122.43576049804686",
                            "37.70660472542312, -122.43301391601562",
                            "37.712059855877314, -122.36434936523438",
                            "37.75350561243041, -122.37396240234374",
                        ],
                    ),
                    Geometry(
                        type="CIRCLE",
                        position="37.71205, -121.36434",
                        radius=6000,
                    ),
                    Geometry(
                        type="CIRCLE",
                        position="37.31205, -121.36434",
                        radius=1000,
                    ),
                ],
            ),
        )

        print(f"\nThere are {response.summary.totalResults} pizza restaurants in the given geometries")


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()

    asyncio.run(get_place_by_id(user_api_key))
    asyncio.run(get_nearby_search(user_api_key))
    asyncio.run(post_geometry_search(user_api_key))



"""Premium Geocoding tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.places import PremiumGeocodingApi
from tomtom_apis.places.models import ResultType, SearchResponse


@pytest.fixture(name="premium_geocoding_api")
async def fixture_premium_geocoding_api() -> AsyncGenerator[PremiumGeocodingApi]:
    """Fixture for PremiumGeocodingApi."""
    options = ApiOptions(api_key=API_KEY)
    async with PremiumGeocodingApi(options) as premium_geocoding:
        yield premium_geocoding


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/premium_geocoding/get_geocode.json"], indirect=True)
async def test_deserialization_get_geocode(premium_geocoding_api: PremiumGeocodingApi) -> None:
    """Test the get_geocode method."""
    response = await premium_geocoding_api.get_geocode(
        query="De Ruijterkade 154 Amsterdam",
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert len(response.results) == 2
    assert response.results[0]
    assert response.results[0].type
    assert response.results[0].type == ResultType.POINT_ADDRESS
    assert response.results[0].position
    assert response.results[0].position.lat == 30.23966941103544
    assert response.results[0].position.lon == -97.78704138350255



"""Conftest for the tests."""

from pathlib import Path

import pytest
from aresponses import ResponsesMockServer

from tomtom_apis.const import HttpStatus

from .const import DEFAULT_HEADERS


def load_json(filename: str) -> str:
    """Load a fixture."""
    path = Path(__package__) / "fixtures" / filename
    return path.read_text(encoding="utf-8")


def load_png(filename: str) -> bytes:
    """Load a fixture."""
    path = Path(__package__) / "fixtures" / filename
    return path.read_bytes()


@pytest.fixture(name="json_response")
def fixture_json_response(request: pytest.FixtureRequest, aresponses: ResponsesMockServer) -> None:
    """Fixture for adding the aresponses response with a configurable JSON file."""
    fixture_filename = request.param
    aresponses.add(
        response=aresponses.Response(
            status=HttpStatus.OK,
            headers=DEFAULT_HEADERS,
            text=load_json(fixture_filename),
        ),
    )


@pytest.fixture(name="image_response")
def fixture_image_response(request: pytest.FixtureRequest, aresponses: ResponsesMockServer) -> None:
    """Fixture for adding the aresponses response with a configurable image file."""
    fixture_filename = request.param
    aresponses.add(
        response=aresponses.Response(
            status=HttpStatus.OK,
            content_type="image/png",
            text=load_png(fixture_filename).decode("latin1"),
        ),
    )



"""Landmark routing example."""

# pylint: disable=duplicate-code

import asyncio
import itertools
import os

from attr import dataclass

from tomtom_apis import ApiOptions
from tomtom_apis.models import LatLon, LatLonList, TravelModeType
from tomtom_apis.routing import RoutingApi
from tomtom_apis.routing.models import CalculateRouteParams, RouteType


@dataclass(kw_only=True)
class Landmark:
    """Representation of a landmark location."""

    name: str
    location: LatLon


# Top 5 US landmarks, according to ChatGPT.
LANDMARKS: list[Landmark] = [
    Landmark(name="Statue of Liberty", location=LatLon(lat=40.6892, lon=-74.0445)),
    Landmark(name="Grand Canyon", location=LatLon(lat=36.1069, lon=-112.1129)),
    Landmark(name="Mount Rushmore National Memorial", location=LatLon(lat=43.8791, lon=-103.4591)),
    Landmark(name="Golden Gate Bridge", location=LatLon(lat=37.8199, lon=-122.4783)),
    Landmark(name="Times Square", location=LatLon(lat=40.7580, lon=-73.9855)),
]


def meters_to_km(meters: int) -> int:
    """Helper function for outputting kilometers."""
    kilometers = meters // 1000
    return int(kilometers)


def seconds_to_minutes(seconds: int) -> int:
    """Helper function for outputting minutes."""
    minutes = round(seconds / 60)
    return int(minutes)


def seconds_to_hours_minutes(seconds: int) -> str:
    """Helper function for outputting travel time."""
    total_minutes = round(seconds / 60)
    hours = total_minutes // 60
    minutes = total_minutes % 60
    return f"{hours} hours {minutes} minutes"


async def plan_route(routing_api: RoutingApi, start: Landmark, destination: Landmark) -> None:
    """Plan a route from one landmark to another."""
    print(f"\nPlanning a route from {start.name} to {destination.name}:")

    response = await routing_api.get_calculate_route(
        locations=LatLonList(locations=[start.location, destination.location]),
        params=CalculateRouteParams(
            maxAlternatives=0,
            routeType=RouteType.FASTEST,
            traffic=True,
            travelMode=TravelModeType.CAR,
        ),
    )

    length = response.routes[0].summary.lengthInMeters
    time = response.routes[0].summary.travelTimeInSeconds
    traffic_time = response.routes[0].summary.trafficDelayInSeconds

    print(f"{meters_to_km(length)}km with a travel time {seconds_to_hours_minutes(time)}, currently {seconds_to_minutes(traffic_time)} minutes delay")


async def routes(api_key: str) -> None:
    """Generate all unique combinations of 2 landmarks and plan a route between them."""
    route_combinations = list(itertools.combinations(LANDMARKS, 2))

    options = ApiOptions(api_key=api_key)
    async with RoutingApi(options) as routing_api:
        for combination in route_combinations:
            await plan_route(routing_api, combination[0], combination[1])


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()
    asyncio.run(routes(user_api_key))



"""Examples of some maps API calls."""

# pylint: disable=duplicate-code

import asyncio
import os

from tomtom_apis import ApiOptions
from tomtom_apis.maps.map_display import MapDisplayApi


async def map_copyrights(api_key: str) -> None:
    """Example for map_copyrights."""
    async with MapDisplayApi(ApiOptions(api_key=api_key, gzip_compression=True, tracking_id=True)) as map_display_api:
        response = await map_display_api.get_map_copyrights()

        print(f"\nMap copyrights: \n{response}")


async def get_map_service_copyrights(api_key: str) -> None:
    """Example for get_map_service_copyrights."""
    async with MapDisplayApi(ApiOptions(api_key=api_key, gzip_compression=True, tracking_id=True)) as map_display_api:
        response = await map_display_api.get_map_service_copyrights()

        print(f"\nMap service copyrights: \n{response}")


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()

    asyncio.run(map_copyrights(user_api_key))
    asyncio.run(get_map_service_copyrights(user_api_key))



"""Constants for testing."""

from tomtom_apis.models import LatLon

API_KEY: str = "abcdef123456"
DEFAULT_HEADERS: dict[str, str] = {"Content-Type": "application/json", "User-Agent": "TomTomApiPython/0.0.0"}

LOC_AMSTERDAM: LatLon = LatLon(lat=52.377956, lon=4.897070)
LOC_ROTTERDAM: LatLon = LatLon(lat=51.926517, lon=4.462456)



"""Examples of geocoding API calls."""

# pylint: disable=duplicate-code

import asyncio
import os

from tomtom_apis import ApiOptions
from tomtom_apis.places import GeocodingApi


async def get_geocode(api_key: str) -> None:
    """Example for get_geocode."""
    query = "De Ruijterkade 154 Amsterdam"
    async with GeocodingApi(ApiOptions(api_key=api_key)) as geo_coding_api:
        response = await geo_coding_api.get_geocode(query=query)

        print(f"\nGeocode for '{query}' = {response.results[0].type} @ {response.results[0].position.lat},{response.results[0].position.lon}")


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()

    asyncio.run(get_geocode(user_api_key))



"""Reverse Geocoding tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.models import LatLon, ViewType
from tomtom_apis.places import ReverseGeocodingApi
from tomtom_apis.places.models import CrossStreetLookupParams, ReverseGeocodeParams, ReverseGeocodeResponse


@pytest.fixture(name="reverse_geocoding_api")
async def fixture_reverse_geocoding_api() -> AsyncGenerator[ReverseGeocodingApi]:
    """Fixture for ReverseGeocodingApi."""
    options = ApiOptions(api_key=API_KEY)
    async with ReverseGeocodingApi(options) as reverse_geocoding:
        yield reverse_geocoding


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/reverse_geocoding/get_reverse_geocode.json"], indirect=True)
async def test_deserialization_get_reverse_geocode(reverse_geocoding_api: ReverseGeocodingApi) -> None:
    """Test the get_reverse_geocode method."""
    response = await reverse_geocoding_api.get_reverse_geocode(
        position=LatLon(
            lat=37.8328,
            lon=-122.27669,
        ),
        params=ReverseGeocodeParams(
            returnSpeedLimit=False,
            radius=10000,
            allowFreeformNewLine=False,
            returnMatchType=False,
            view=ViewType.UNIFIED,
        ),
    )

    assert response
    assert isinstance(response, ReverseGeocodeResponse)
    assert response.addresses
    assert response.addresses[0]
    assert response.addresses[0].address
    assert response.addresses[0].address.country
    assert response.addresses[0].address.country == "United States"


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/reverse_geocoding/get_cross_street_lookup.json"], indirect=True)
async def test_deserialization_get_cross_street_lookup(reverse_geocoding_api: ReverseGeocodingApi) -> None:
    """Test the get_cross_street_lookup method."""
    response = await reverse_geocoding_api.get_cross_street_lookup(
        position=LatLon(
            lat=37.8328,
            lon=-122.27669,
        ),
        params=CrossStreetLookupParams(
            limit=1,
            radius=10000,
            allowFreeformNewLine=False,
            view=ViewType.UNIFIED,
        ),
    )

    assert response
    assert isinstance(response, ReverseGeocodeResponse)
    assert response.addresses
    assert response.addresses[0]
    assert response.addresses[0].address
    assert response.addresses[0].address.countrySubdivisionName == "California"



"""Examples of reverse geocoding API calls."""

# pylint: disable=duplicate-code

import asyncio
import os

from tomtom_apis import ApiOptions
from tomtom_apis.models import Language, LatLon
from tomtom_apis.places import ReverseGeocodingApi
from tomtom_apis.places.models import ReverseGeocodeParams


async def get_reverse_geocode(api_key: str) -> None:
    """Example for get_reverse_geocode."""
    position = LatLon(lat=48.858093, lon=2.294694)
    async with ReverseGeocodingApi(ApiOptions(api_key=api_key)) as geo_coding_api:
        response = await geo_coding_api.get_reverse_geocode(
            position=position,
            params=ReverseGeocodeParams(language=Language.EN_GB),
        )

        print(f"\nReverse geocode for '{position}' = {response.addresses[0].address.freeformAddress}")


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()

    asyncio.run(get_reverse_geocode(user_api_key))



"""Test utils."""

import math
import re
from enum import Enum, IntEnum, StrEnum

import pytest

from tomtom_apis.exceptions import RangeExceptionError
from tomtom_apis.models import LatLon, MapTile
from tomtom_apis.utils import lat_lon_to_tile_zxy, serialize_bool, serialize_enum, serialize_list, serialize_list_brackets, tile_zxy_to_lat_lon


def test_lat_lon_to_tile_zxy_valid() -> None:
    """Test valid lat_lon_to_tile_zxy input."""
    result = lat_lon_to_tile_zxy(37.7749, -122.4194, 10)
    assert isinstance(result, MapTile)
    assert result.zoom == 10
    assert result.x == 163
    assert result.y == 395


def test_lat_lon_to_tile_zxy_boundary() -> None:
    """Test boundary conditions for lat_lon_to_tile_zxy."""
    result = lat_lon_to_tile_zxy(85.051128779806, 180.0, 10)
    assert isinstance(result, MapTile)
    assert result.zoom == 10
    assert result.x == 1024
    assert result.y == 0


def test_lat_lon_to_tile_zxy_invalid_zoom_level() -> None:
    """Test invalid zoom level for lat_lon_to_tile_zxy."""
    with pytest.raises(RangeExceptionError, match=re.escape("zoom_level value is out of range [0, 22]")):
        lat_lon_to_tile_zxy(37.7749, -122.4194, 23)


def test_lat_lon_to_tile_zxy_invalid_latitude() -> None:
    """Test invalid latitude for lat_lon_to_tile_zxy."""
    with pytest.raises(RangeExceptionError, match=re.escape("lat value is out of range [-85.051128779807, 85.051128779806]")):
        lat_lon_to_tile_zxy(90.0, -122.4194, 10)


def test_lat_lon_to_tile_zxy_invalid_longitude() -> None:
    """Test invalid longitude for lat_lon_to_tile_zxy."""
    with pytest.raises(RangeExceptionError, match=re.escape("lon value is out of range [-180.0, 180.0]")):
        lat_lon_to_tile_zxy(37.7749, -200.0, 10)


def test_tile_zxy_to_lat_lon_valid() -> None:
    """Test valid tile_zxy_to_lat_lon input."""
    result = tile_zxy_to_lat_lon(10, 163, 395)
    assert isinstance(result, LatLon)
    assert math.isclose(result.lat, 37.7749, rel_tol=1e-2)
    assert math.isclose(result.lon, -122.4194, rel_tol=1e-2)


def test_tile_zxy_to_lat_lon_invalid_zoom_level() -> None:
    """Test invalid zoom level for tile_zxy_to_lat_lon."""
    with pytest.raises(RangeExceptionError, match=re.escape("zoom_level value is out of range [0, 22]")):
        tile_zxy_to_lat_lon(23, 163, 395)


def test_tile_zxy_to_lat_lon_invalid_x() -> None:
    """Test invalid x coordinate for tile_zxy_to_lat_lon."""
    with pytest.raises(RangeExceptionError, match=re.escape("x value is out of range [0, 1023]")):
        tile_zxy_to_lat_lon(10, -1, 395)


def test_tile_zxy_to_lat_lon_invalid_y() -> None:
    """Test invalid y coordinate for tile_zxy_to_lat_lon."""
    with pytest.raises(RangeExceptionError, match=re.escape("y value is out of range [0, 1023]")):
        tile_zxy_to_lat_lon(10, 163, -1)


def test_serialize_bool() -> None:
    """Test cases for test_serialize_bool."""
    # Test with True
    assert serialize_bool(x=True) == "true"
    # Test with False
    assert serialize_bool(x=False) == "false"


class Color(Enum):
    """Simple enum for testing."""

    RED = "red"
    GREEN = "green"
    BLUE = "blue"


class StrColor(StrEnum):
    """Simple string enum for testing."""

    RED = "red"
    GREEN = "green"
    BLUE = "blue"


class IntColor(IntEnum):
    """Simple int enum for testing."""

    RED = 1
    GREEN = 2
    BLUE = 3


def test_serialize_enum() -> None:
    """Test cases for serialize_enum."""
    assert serialize_enum(Color.RED) == "red"
    assert serialize_enum(StrColor.RED) == "red"
    assert serialize_enum(IntColor.RED) == "1"


def test_serialize_list() -> None:
    """Test cases for test_serialize_list."""
    # Test with an empty list
    assert serialize_list([]) is None
    # Test with a list of integers
    assert serialize_list([1, 2, 3]) == "1,2,3"
    # Test with a list of strings
    assert serialize_list(["a", "b", "c"]) == "a,b,c"
    # Test with a mixed list
    assert serialize_list([1, "b", 3.0, True]) == "1,b,3.0,true"
    assert serialize_list([False, "yes", 10]) == "false,yes,10"
    assert serialize_list(["True", False]) == "True,false"
    # Test with a list containing Enums
    assert serialize_list([Color.RED, Color.GREEN, Color.BLUE]) == "red,green,blue"
    assert serialize_list([StrColor.RED, StrColor.GREEN, StrColor.BLUE]) == "red,green,blue"
    assert serialize_list([IntColor.RED, IntColor.GREEN, IntColor.BLUE]) == "1,2,3"
    assert serialize_list([Color.RED, StrColor.GREEN, IntColor.BLUE]) == "red,green,3"
    assert serialize_list([StrColor.RED, 1, True]) == "red,1,true"


def test_serialize_list_brackets() -> None:
    """Test cases for test_serialize_list_brackets."""
    # Test with an empty list
    assert serialize_list_brackets([]) is None
    # Test with a list of integers
    assert serialize_list_brackets([1, 2, 3]) == "[1,2,3]"
    # Test with a list of strings
    assert serialize_list_brackets(["a", "b", "c"]) == "[a,b,c]"
    # Test with a mixed list
    assert serialize_list_brackets([1, "b", 3.0, True]) == "[1,b,3.0,true]"
    assert serialize_list_brackets([False, "yes", 10]) == "[false,yes,10]"
    assert serialize_list_brackets(["True", False]) == "[True,false]"
    # Test with a list containing Enums
    assert serialize_list_brackets([Color.RED, Color.GREEN, Color.BLUE]) == "[red,green,blue]"
    assert serialize_list_brackets([StrColor.RED, StrColor.GREEN, StrColor.BLUE]) == "[red,green,blue]"
    assert serialize_list_brackets([IntColor.RED, IntColor.GREEN, IntColor.BLUE]) == "[1,2,3]"
    assert serialize_list_brackets([Color.RED, StrColor.GREEN, IntColor.BLUE]) == "[red,green,3]"
    assert serialize_list_brackets([StrColor.RED, 1, True]) == "[red,1,true]"



"""Auto Stream tests."""

import pytest

from tomtom_apis.automotive import AutoStreamApi


def test_api_not_implemented() -> None:
    """Test API not implemented yet."""
    with pytest.raises(NotImplementedError):
        AutoStreamApi()



"""Intermediate Traffic tests."""

import pytest

from tomtom_apis.traffic import IntermediateTrafficApi


def test_api_not_implemented() -> None:
    """Test API not implemented yet."""
    with pytest.raises(NotImplementedError):
        IntermediateTrafficApi()



"""Traffic tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.exceptions import MutualExclusiveParamsError
from tomtom_apis.models import Language, TileSizeType
from tomtom_apis.traffic import TrafficApi
from tomtom_apis.traffic.models import (
    BBoxParam,
    BoudingBoxParam,
    CategoryFilterType,
    FlowSegmentDataParams,
    FlowSegmentDataResponse,
    FlowStyleType,
    FlowTagType,
    FlowType,
    IncidentDetailsParams,
    IncidentDetailsPostData,
    IncidentDetailsResponse,
    IncidentStyleType,
    IncidentTileFormatType,
    IncidentViewportResponse,
    RasterFlowTilesParams,
    RasterIncidentTilesParams,
    SpeedUnitType,
    TimeValidityFilterType,
    VectorFlowTilesParams,
    VectorIncidentTilesParams,
)


@pytest.fixture(name="traffic_api")
async def fixture_traffic_api() -> AsyncGenerator[TrafficApi]:
    """Fixture for TrafficApi."""
    options = ApiOptions(api_key=API_KEY)
    async with TrafficApi(options) as traffic:
        yield traffic


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["traffic/traffic/get_incident_details_bbox.json"], indirect=True)
async def test_deserialization_get_incident_details_bbox(traffic_api: TrafficApi) -> None:
    """Test the get_incident_details method."""
    response = await traffic_api.get_incident_details(
        bbox=BBoxParam(
            minLon=4.8854592519716675,
            minLat=52.36934334773164,
            maxLon=4.897883244144765,
            maxLat=52.37496348620152,
        ),
        params=IncidentDetailsParams(
            fields="{incidents{type,geometry{type,coordinates},properties{iconCategory}}}",
            language=Language.EN_GB,
            categoryFilter=[
                CategoryFilterType.UNKNOWN,
                CategoryFilterType.ACCIDENT,
                CategoryFilterType.FOG,
                CategoryFilterType.DANGEROUS_CONDITIONS,
                CategoryFilterType.RAIN,
                CategoryFilterType.ICE,
                CategoryFilterType.JAM,
                CategoryFilterType.LANE_CLOSED,
                CategoryFilterType.ROAD_CLOSED,
                CategoryFilterType.ROAD_WORKS,
                CategoryFilterType.WIND,
                CategoryFilterType.FLOODING,
                CategoryFilterType.BROKEN_DOWN_VEHICLE,
            ],
            timeValidityFilter=[TimeValidityFilterType.PRESENT],
        ),
    )

    assert response
    assert isinstance(response, IncidentDetailsResponse)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["traffic/traffic/get_incident_details_ids.json"], indirect=True)
async def test_deserialization_get_incident_details_ids(traffic_api: TrafficApi) -> None:
    """Test the get_incident_details method."""
    response = await traffic_api.get_incident_details(
        ids=["string"],
        params=IncidentDetailsParams(
            fields="{incidents{type,geometry{type,coordinates},properties{iconCategory}}}",
            language=Language.EN_GB,
            categoryFilter=[
                CategoryFilterType.UNKNOWN,
                CategoryFilterType.ACCIDENT,
                CategoryFilterType.FOG,
                CategoryFilterType.DANGEROUS_CONDITIONS,
                CategoryFilterType.RAIN,
                CategoryFilterType.ICE,
                CategoryFilterType.JAM,
                CategoryFilterType.LANE_CLOSED,
                CategoryFilterType.ROAD_CLOSED,
                CategoryFilterType.ROAD_WORKS,
                CategoryFilterType.WIND,
                CategoryFilterType.FLOODING,
                CategoryFilterType.BROKEN_DOWN_VEHICLE,
            ],
            timeValidityFilter=[TimeValidityFilterType.PRESENT],
        ),
    )

    assert response
    assert isinstance(response, IncidentDetailsResponse)


async def test_get_incident_details_mutually_exclusive_parameters(traffic_api: TrafficApi) -> None:
    """Test the mutually exclusive parameters from the get_incident_details method."""
    with pytest.raises(MutualExclusiveParamsError, match="Mutually exclusive parameters provided: bbox, ids"):
        await traffic_api.get_incident_details(
            params=IncidentDetailsParams(),
        )


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["traffic/traffic/post_incident_details.json"], indirect=True)
async def test_deserialization_post_incident_details(traffic_api: TrafficApi) -> None:
    """Test the post_incident_details method."""
    response = await traffic_api.post_incident_details(
        params=IncidentDetailsParams(
            fields="{incidents{type,geometry{type,coordinates},properties{iconCategory}}}",
            language=Language.EN_GB,
            categoryFilter=[
                CategoryFilterType.UNKNOWN,
                CategoryFilterType.ACCIDENT,
                CategoryFilterType.FOG,
                CategoryFilterType.DANGEROUS_CONDITIONS,
                CategoryFilterType.RAIN,
                CategoryFilterType.ICE,
                CategoryFilterType.JAM,
                CategoryFilterType.LANE_CLOSED,
                CategoryFilterType.ROAD_CLOSED,
                CategoryFilterType.ROAD_WORKS,
                CategoryFilterType.WIND,
                CategoryFilterType.FLOODING,
                CategoryFilterType.BROKEN_DOWN_VEHICLE,
            ],
            timeValidityFilter=[TimeValidityFilterType.PRESENT],
        ),
        data=IncidentDetailsPostData(ids=["string"]),
    )

    assert response
    assert isinstance(response, IncidentDetailsResponse)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["traffic/traffic/get_incident_viewport.json"], indirect=True)
async def test_deserialization_get_incident_viewport(traffic_api: TrafficApi) -> None:
    """Test the get_incident_viewport method."""
    response = await traffic_api.get_incident_viewport(
        bounding_box=BoudingBoxParam(minY=-939584.4813015489, minX=-23954526.723651607, maxY=14675583.153020501, maxX=25043442.895825107),
        bounding_zoom=2,
        overview_box=BoudingBoxParam(minY=-939584.4813015489, minX=-23954526.723651607, maxY=14675583.153020501, maxX=25043442.895825107),
        overview_zoom=2,
        copyright_information=True,
    )

    assert response
    assert isinstance(response, IncidentViewportResponse)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["traffic/traffic/get_raster_incident_tile.png"], indirect=True)
async def test_deserialization_get_static_image(traffic_api: TrafficApi) -> None:
    """Test the get_static_image method."""
    response = await traffic_api.get_raster_incident_tile(
        style=IncidentStyleType.S0,
        x=1207,
        y=1539,
        zoom=12,
        image_format=IncidentTileFormatType.PNG,
        params=RasterIncidentTilesParams(t="-1", tileSize=TileSizeType.SIZE_256),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["traffic/traffic/get_vector_incident_tile.pbf"], indirect=True)
async def test_deserialization_get_tile_v1(traffic_api: TrafficApi) -> None:
    """Test the get_vector_incident_tile method."""
    response = await traffic_api.get_vector_incident_tile(
        x=1207,
        y=1539,
        zoom=12,
        params=VectorIncidentTilesParams(
            t="-1",
            tags=[
                "icon_category",
                "description",
                "delay",
                "left_hand_traffic",
                "magnitude",
                "traffic_road_coverage",
                "end_date",
                "id",
                "road_category",
                "road_subcategory",
            ],
            language=Language.EN_GB,
        ),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["traffic/traffic/get_flow_segment_data.json"], indirect=True)
async def test_deserialization_get_flow_segment_data(traffic_api: TrafficApi) -> None:
    """Test the get_flow_segment_data method."""
    response = await traffic_api.get_flow_segment_data(
        style=FlowStyleType.RELATIVE0,
        zoom=10,
        point="52.41072,4.84239",
        params=FlowSegmentDataParams(
            unit=SpeedUnitType.KMPH,
            openLr=False,
        ),
    )

    assert response
    assert isinstance(response, FlowSegmentDataResponse)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["traffic/traffic/get_raster_flow_tiles.pbf"], indirect=True)
async def test_deserialization_get_raster_flow_tiles(traffic_api: TrafficApi) -> None:
    """Test the get_raster_flow_tiles method."""
    response = await traffic_api.get_raster_flow_tiles(
        style=FlowStyleType.RELATIVE0,
        x=1207,
        y=1539,
        zoom=12,
        params=RasterFlowTilesParams(
            tileSize=TileSizeType.SIZE_256,
        ),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["traffic/traffic/get_vector_flow_tiles.pbf"], indirect=True)
async def test_deserialization_get_vector_flow_tiles(traffic_api: TrafficApi) -> None:
    """Test the get_vector_flow_tiles method."""
    response = await traffic_api.get_vector_flow_tiles(
        flow_type=FlowType.RELATIVE,
        x=1207,
        y=1539,
        zoom=12,
        params=VectorFlowTilesParams(
            margin=0.1,
            tags=[
                FlowTagType.TRAFFIC_LEVEL,
                FlowTagType.TRAFFIC_ROAD_COVERAGE,
                FlowTagType.LEFT_HAND_TRAFFIC,
                FlowTagType.ROAD_CLOSURE,
                FlowTagType.ROAD_CATEGORY,
                FlowTagType.ROAD_SUBCATEGORY,
            ],
        ),
    )

    assert response
    assert isinstance(response, bytes)



"""Test for the Api."""

import socket
from collections.abc import AsyncGenerator
from dataclasses import dataclass
from unittest.mock import AsyncMock, patch

import orjson
import pytest
from aiohttp import ClientConnectionError, ClientError, ClientResponse, ClientResponseError, ClientSession, RequestInfo
from mashumaro.mixins.orjson import DataClassORJSONMixin
from multidict import CIMultiDict, CIMultiDictProxy
from yarl import URL

from tomtom_apis.api import ApiOptions, BaseApi, BasePostData, Response
from tomtom_apis.const import TRACKING_ID_HEADER, HttpMethod, HttpStatus
from tomtom_apis.exceptions import TomTomAPIClientError, TomTomAPIConnectionError, TomTomAPIError, TomTomAPIRequestTimeoutError, TomTomAPIServerError

from .const import API_KEY


@dataclass(kw_only=True)
class MockModel(DataClassORJSONMixin):
    """Mock model."""

    key: str


@pytest.fixture(name="mock_response")
def fixture_mock_response() -> AsyncMock:
    """Fixture for mock response."""
    mock_resp = AsyncMock(spec=ClientResponse)
    mock_resp.status = HttpStatus.OK
    mock_resp.headers = {
        "content-type": "application/json;charset=utf-8",
        "Tracking-ID": "1234567890",
        "x-tomtom-processed-by": "westeurope",
    }
    mock_resp.text = AsyncMock(return_value='{"key": "value"}')
    return mock_resp


@pytest.fixture(name="mock_session")
def fixture_mock_session(mock_response: AsyncMock) -> AsyncMock:
    """Fixture for mock session."""
    session = AsyncMock(spec=ClientSession)
    session.closed = False

    async def close_mock() -> None:
        session.closed = True

    session.close = AsyncMock(side_effect=close_mock)
    session.request = AsyncMock(return_value=mock_response)

    return session


@pytest.fixture(name="mock_request_info")
def fixture_mock_request_info() -> RequestInfo:
    """Fixture for request info."""
    return RequestInfo(
        url=URL("http://example.com"),
        method=HttpMethod.GET,
        headers=CIMultiDictProxy(CIMultiDict({})),
        real_url=URL("http://example.com"),
    )


@pytest.fixture(name="base_api")
async def fixture_base_api(mock_session: AsyncMock) -> AsyncGenerator[BaseApi]:
    """Fixture for BaseApi."""
    options = ApiOptions(api_key=API_KEY)
    async with BaseApi(options, mock_session) as base:
        yield base


async def test_deserialize_success(mock_response: AsyncMock) -> None:
    """Test the deserialize method."""
    response = Response(mock_response)
    result = await response.deserialize(MockModel)

    assert isinstance(result, MockModel)
    assert result.key == "value"
    mock_response.text.assert_awaited_once()


async def test_deserialize_failure(mock_response: AsyncMock) -> None:
    """Test the deserialize method."""
    mock_response.text.side_effect = Exception("Deserialization error")
    response = Response(mock_response)

    with pytest.raises(Exception, match="Deserialization error"):
        await response.deserialize(MockModel)

    mock_response.text.assert_awaited_once()


async def test_dict_success(mock_response: AsyncMock) -> None:
    """Test the dict method."""
    response = Response(mock_response)
    result = await response.dict()

    assert result == {"key": "value"}
    mock_response.text.assert_awaited_once()


async def test_dict_json_decode_error(mock_response: AsyncMock) -> None:
    """Test the dict method."""
    mock_response.text.return_value = "invalid json"
    response = Response(mock_response)

    with pytest.raises(orjson.JSONDecodeError):  # pylint: disable=maybe-no-member
        await response.dict()

    mock_response.text.assert_awaited_once()


async def test_text(mock_response: AsyncMock) -> None:
    """Test the text method."""
    mock_response.text.return_value = "response text"
    response = Response(mock_response)
    result = await response.text()

    assert result == "response text"
    mock_response.text.assert_awaited_once()


async def test_get_request(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the get method."""
    endpoint = "/test/endpoint"
    response = await base_api.get(endpoint)

    mock_session.request.assert_called_once_with(
        HttpMethod.GET,
        endpoint,
        params={"key": API_KEY},
        json=None,
        headers={
            "Content-Type": "application/json",
            "User-Agent": "python/tomtom_apis",
        },
    )
    assert isinstance(response, Response)
    assert response.status == HttpStatus.OK


async def test_get_request_with_gzip(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the get method with gzip."""
    endpoint = "/test/endpoint"
    base_api.options.gzip_compression = True
    response = await base_api.get(endpoint)

    mock_session.request.assert_called_once_with(
        HttpMethod.GET,
        endpoint,
        params={"key": API_KEY},
        json=None,
        headers={
            "Content-Type": "application/json",
            "User-Agent": "python/tomtom_apis",
            "Accept-Encoding": "gzip",
        },
    )
    assert isinstance(response, Response)
    assert response.status == HttpStatus.OK


async def test_post_request(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the post method."""
    endpoint = "/test/endpoint"
    data = BasePostData()
    response = await base_api.post(endpoint, data=data)

    mock_session.request.assert_called_once_with(
        HttpMethod.POST,
        endpoint,
        params={"key": API_KEY},
        json=data.to_dict(),
        headers={
            "Content-Type": "application/json",
            "User-Agent": "python/tomtom_apis",
        },
    )
    assert isinstance(response, Response)
    assert response.status == HttpStatus.OK


async def test_delete_request(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the delete method."""
    endpoint = "/test/endpoint"
    response = await base_api.delete(endpoint)

    mock_session.request.assert_called_once_with(
        HttpMethod.DELETE,
        endpoint,
        params={"key": API_KEY},
        json=None,
        headers={
            "Content-Type": "application/json",
            "User-Agent": "python/tomtom_apis",
        },
    )
    assert isinstance(response, Response)
    assert response.status == HttpStatus.OK


async def test_put_request(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the put method."""
    endpoint = "/test/endpoint"
    data = BasePostData()
    response = await base_api.put(endpoint, data=data)

    mock_session.request.assert_called_once_with(
        HttpMethod.PUT,
        endpoint,
        params={"key": API_KEY},
        json=data.to_dict(),
        headers={
            "Content-Type": "application/json",
            "User-Agent": "python/tomtom_apis",
        },
    )
    assert isinstance(response, Response)
    assert response.status == HttpStatus.OK


async def test_request_timeout(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the request method with a timeout."""
    mock_session.request.side_effect = TimeoutError()
    with pytest.raises(TomTomAPIRequestTimeoutError):
        await base_api.get("/timeout/endpoint")


async def test_request_connection_error(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the request method with a connection error."""
    mock_session.request.side_effect = ClientConnectionError()
    with pytest.raises(TomTomAPIConnectionError):
        await base_api.get("/connection/error")


async def test_request_client_response_error(base_api: BaseApi, mock_session: AsyncMock, mock_request_info: RequestInfo) -> None:
    """Test the request method with a client response error."""
    error = ClientResponseError(request_info=mock_request_info, history=(), status=HttpStatus.BAD_REQUEST)
    mock_session.request.side_effect = error
    with pytest.raises(TomTomAPIClientError):
        await base_api.get("/client/error")


async def test_request_server_response_error(base_api: BaseApi, mock_session: AsyncMock, mock_request_info: RequestInfo) -> None:
    """Test the request method with a server response error."""
    error = ClientResponseError(request_info=mock_request_info, history=(), status=HttpStatus.INTERNAL_SERVER_ERROR)
    mock_session.request.side_effect = error
    with pytest.raises(TomTomAPIServerError):
        await base_api.get("/server/error")


async def test_request_unknown_response_error(base_api: BaseApi, mock_session: AsyncMock, mock_request_info: RequestInfo) -> None:
    """Test the request method with an unknown response error."""
    error = ClientResponseError(request_info=mock_request_info, history=(), status=HttpStatus.UNASSIGNED)
    mock_session.request.side_effect = error
    with pytest.raises(TomTomAPIError):
        await base_api.get("/server/error")


async def test_request_client_error(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the request method with a client error."""
    error = ClientError()
    mock_session.request.side_effect = error
    with pytest.raises(TomTomAPIConnectionError):
        await base_api.get("/server/error")


async def test_request_socket_error(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the request method with a socket error."""
    error = socket.gaierror()
    mock_session.request.side_effect = error
    with pytest.raises(TomTomAPIConnectionError):
        await base_api.get("/server/error")


async def test_tracking_id(base_api: BaseApi, mock_session: AsyncMock) -> None:
    """Test the tracking_id option."""
    base_api.options.tracking_id = True
    with patch("uuid.uuid4", return_value="mock-uuid"):
        endpoint = "/test/endpoint"
        response = await base_api.get(endpoint)

        assert mock_session.request.call_args[1]["headers"][TRACKING_ID_HEADER] == "mock-uuid"
        assert isinstance(response, Response)
        assert response.status == HttpStatus.OK


async def test_manual_session_close(mock_session: AsyncMock) -> None:
    """Test manual closing of the provided session."""
    options = ApiOptions(api_key=API_KEY)
    async with BaseApi(options, mock_session) as base_api:
        assert not base_api._close_session  # pylint: disable=protected-access
        assert not base_api.session.closed

    await base_api.close()
    assert base_api.session.closed


async def test_auto_session_close() -> None:
    """Test auto closing of the default session."""
    options = ApiOptions(api_key=API_KEY)
    async with BaseApi(options) as base_api:
        assert base_api._close_session  # pylint: disable=protected-access
        assert not base_api.session.closed

    assert base_api.session.closed



"""Exceptions for the TomTom API."""


class TomTomAPIError(Exception):
    """Base exception for all errors raised by the TomTom SDK."""


class TomTomAPIClientError(TomTomAPIError):
    """Exception raised for client-side errors (4xx)."""


class TomTomAPIServerError(TomTomAPIError):
    """Exception raised for server-side errors (5xx)."""


class TomTomAPIConnectionError(TomTomAPIError):
    """Exception raised for connection errors."""


class TomTomAPIRequestTimeoutError(TomTomAPIError):
    """Exception raised for request timeouts."""


class RangeExceptionError(Exception):
    """Exception raised when a value is out of range."""

    def __init__(self, field: str, min_number: float, max_number: float) -> None:
        """Initialize the RangeExceptionError."""
        super().__init__(f"{field} value is out of range [{min_number}, {max_number}]")


class MutualExclusiveParamsError(Exception):
    """Exception raised when mutually exclusive parameters are provided."""

    def __init__(self, params: list[str]) -> None:
        """Initialize the MutualExclusiveParamsError."""
        super().__init__(f"Mutually exclusive parameters provided: {', '.join(params)}")



"""Update test fixtures.

This script downloads and replaces fixtures with the latest content from the real API.
The idea is to do this regularly to keep the fixtures up-to-date and detect breaking changes.
"""

import asyncio
import json
import os
import subprocess
from pathlib import Path
from typing import Any

import aiohttp

BASE_URL = "https://api.tomtom.com"
PROJECT_ROOT = Path(__file__).parent.parent
FIXTURES_PATH = Path(PROJECT_ROOT) / "tests" / "fixtures"
FIXTURE_FILE_PATH = Path(PROJECT_ROOT) / "scripts" / "data" / "fixtures.json"


class ContentExceptionError(Exception):
    """Exception raised when the content is not as expected."""

    def __init__(self) -> None:
        """Initialize the ContentException."""
        super().__init__("Content must be either a string or bytes")


async def read_json(file_path: Path) -> list[dict[str, Any]]:
    """Read a json file and return its content as a dict."""
    with file_path.open(encoding="UTF-8") as file:
        return json.load(file)


async def make_request(session: aiohttp.ClientSession, method: str, url: str, data: dict | None = None) -> str | bytes | None:
    """Make a request to the TomTom API."""
    try:
        async with session.request(method.upper(), url, json=data) as response:
            response.raise_for_status()
            if "image" in response.content_type:
                return await response.read()
            return await response.text()
    except aiohttp.ClientError as e:
        print(f"Request failed: {e}")
        return None


async def save_fixture(content: str | bytes, fixture_path: Path) -> None:
    """Save a fixture to the specified path."""
    fixture_path.parent.mkdir(parents=True, exist_ok=True)
    if isinstance(content, str):
        mode = "w"
        encoding = "utf-8"
    elif isinstance(content, bytes):
        mode = "wb"
        encoding = None
    else:
        raise ContentExceptionError

    with fixture_path.open(mode=mode, encoding=encoding) as file:
        file.write(content)


async def process_fixture(session: aiohttp.ClientSession, api_entry: dict, api_key: str) -> None:
    """Process a single fixture."""
    for fixture in api_entry["fixtures"]:
        method = fixture.get("method")
        url = fixture.get("url")
        fixture_filename = fixture.get("fixture")
        post_data = fixture.get("data")

        fixture_path = FIXTURES_PATH / fixture_filename
        full_url = f"{BASE_URL}/{url.replace('{{TT_KEY}}', api_key)}"

        if method and url and fixture_path:
            content = await make_request(session, method, full_url, post_data)
            if content:
                await save_fixture(content, fixture_path)
                print(f"Saved fixture to {fixture_path}")
            else:
                print(f"Failed to get content for {fixture_path}")
        else:
            print(f"Invalid fixture: {fixture}")


async def process_fixtures(file_path: Path, api_key: str) -> None:
    """Process fixtures."""
    data = await read_json(file_path)
    async with aiohttp.ClientSession() as session:
        tasks = [process_fixture(session, api_entry, api_key) for api_entry in data]
        await asyncio.gather(*tasks)


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()
    asyncio.run(process_fixtures(FIXTURE_FILE_PATH, user_api_key))

    # Make all fixture files prettier
    subprocess.run(
        [f"{PROJECT_ROOT}/node_modules/.bin/prettier", "--log-level", "silent", "--write", FIXTURES_PATH],
        check=True,
    )



"""Long Distance EV Routing tests."""

from collections.abc import AsyncGenerator
from datetime import datetime

import pytest

from tests.const import API_KEY, LOC_AMSTERDAM
from tomtom_apis.api import ApiOptions
from tomtom_apis.models import LatLonList
from tomtom_apis.routing import LongDistanceEVRoutingApi
from tomtom_apis.routing.models import (
    CalculatedLongDistanceEVRouteResponse,
    CalculateLongDistanceEVRouteParams,
    CalculateLongDistanceEVRoutePostData,
    ChargingConnection,
    ChargingCurve,
    ChargingMode,
    FacilityType,
    PlugType,
    VehicleEngineType,
)


@pytest.fixture(name="long_distance_ev_routing_api")
async def fixture_long_distance_ev_routing_api() -> AsyncGenerator[LongDistanceEVRoutingApi]:
    """Fixture for LongDistanceEVRoutingApi."""
    options = ApiOptions(api_key=API_KEY)
    async with LongDistanceEVRoutingApi(options) as long_distance_ev_routing:
        yield long_distance_ev_routing


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/long_distance_ev_routing/post_calculate_long_distance_ev_route.json"], indirect=True)
async def test_deserialization_post_calculate_long_distance_ev_route(long_distance_ev_routing_api: LongDistanceEVRoutingApi) -> None:
    """Test the post_calculate_long_distance_ev_route method."""
    params = CalculateLongDistanceEVRouteParams(
        vehicleEngineType=VehicleEngineType.ELECTRIC,
        constantSpeedConsumptionInkWhPerHundredkm="50.0,6.5:100.0,8.5",
        currentChargeInkWh=10,
        maxChargeInkWh=40,
        minChargeAtDestinationInkWh=5.2,
        minChargeAtChargingStopsInkWh=1.5,
    )

    data = CalculateLongDistanceEVRoutePostData(
        chargingModes=[
            ChargingMode(
                chargingConnections=[
                    ChargingConnection(
                        facilityType=FacilityType.CHARGE_380_TO_480V_3_PHASE_AT_32A,
                        plugType=PlugType.IEC_62196_TYPE_2_OUTLET,
                    ),
                ],
                chargingCurve=[
                    ChargingCurve(chargeInkWh=6, timeToChargeInSeconds=360),
                    ChargingCurve(chargeInkWh=12, timeToChargeInSeconds=720),
                    ChargingCurve(chargeInkWh=28, timeToChargeInSeconds=1944),
                    ChargingCurve(chargeInkWh=40, timeToChargeInSeconds=4680),
                ],
            ),
            ChargingMode(
                chargingConnections=[
                    ChargingConnection(
                        facilityType=FacilityType.CHARGE_200_TO_240V_1_PHASE_AT_10A,
                        plugType=PlugType.STANDARD_HOUSEHOLD_COUNTRY_SPECIFIC,
                    ),
                ],
                chargingCurve=[
                    ChargingCurve(chargeInkWh=6, timeToChargeInSeconds=15624),
                    ChargingCurve(chargeInkWh=12, timeToChargeInSeconds=32652),
                    ChargingCurve(chargeInkWh=28, timeToChargeInSeconds=76248),
                    ChargingCurve(chargeInkWh=40, timeToChargeInSeconds=109080),
                ],
            ),
        ],
    )

    response = await long_distance_ev_routing_api.post_calculate_long_distance_ev_route(
        locations=LatLonList(locations=[LOC_AMSTERDAM]),
        params=params,
        data=data,
    )

    assert response
    assert isinstance(response, CalculatedLongDistanceEVRouteResponse)
    assert len(response.routes) == 1
    assert response.routes[0].summary
    assert response.routes[0].summary.lengthInMeters > 500000
    assert response.routes[0].summary.travelTimeInSeconds > 10000
    assert response.routes[0].summary.batteryConsumptionInkWh > 30
    assert response.routes[0].summary.remainingChargeAtArrivalInkWh > 4
    assert response.routes[0].summary.totalChargingTimeInSeconds > 3000
    assert isinstance(response.routes[0].summary.departureTime, datetime)
    assert isinstance(response.routes[0].summary.arrivalTime, datetime)
    assert isinstance(response.routes[0].legs[0].summary.departureTime, datetime)
    assert isinstance(response.routes[0].legs[0].summary.arrivalTime, datetime)



"""Waypoint Optimization tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.routing import WaypointOptimizationApi
from tomtom_apis.routing.models import WaypointOptimizationPostData, WaypointOptimizedResponse


@pytest.fixture(name="waypoint_optimization_api")
async def fixture_waypoint_optimization_api() -> AsyncGenerator[WaypointOptimizationApi]:
    """Fixture for WaypointOptimizationApi."""
    options = ApiOptions(api_key=API_KEY)
    async with WaypointOptimizationApi(options) as waypoint_optimization:
        yield waypoint_optimization


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/waypoint_optimization/post_waypointoptimization.json"], indirect=True)
async def test_deserialization_post_waypointoptimization(waypoint_optimization_api: WaypointOptimizationApi) -> None:
    """Test the post_waypointoptimization method."""
    response = await waypoint_optimization_api.post_waypointoptimization(
        data=WaypointOptimizationPostData.from_dict(
            {
                "waypoints": [
                    {"point": {"longitude": 16.90497409165738, "latitude": 52.41111094318538}},
                    {"point": {"longitude": 16.90709715173861, "latitude": 52.410666138418065}},
                    {"point": {"longitude": 16.909298155957515, "latitude": 52.410228259481045}},
                    {"point": {"longitude": 16.907319287272202, "latitude": 52.41061316108531}},
                    {"point": {"longitude": 16.910996727064912, "latitude": 52.41041397793927}},
                    {"point": {"longitude": 16.911123756099897, "latitude": 52.41066703462286}},
                    {"point": {"longitude": 16.90941955426868, "latitude": 52.41122957918256}},
                ],
                "options": {
                    "travelMode": "truck",
                    "vehicleMaxSpeed": 110,
                    "vehicleWeight": 36000,
                    "vehicleAxleWeight": 6000,
                    "vehicleLength": 16.2,
                    "vehicleWidth": 2.4,
                    "vehicleHeight": 3.8,
                    "vehicleCommercial": True,
                    "vehicleLoadType": ["USHazmatClass3", "otherHazmatExplosive"],
                    "vehicleAdrTunnelRestrictionCode": "B",
                },
            },
        ),
    )

    assert response
    assert isinstance(response, WaypointOptimizedResponse)
    assert len(response.optimizedOrder) == 7
    assert response.optimizedOrder == [0, 5, 4, 2, 3, 1, 6]



"""Models for the TomTom Routing API."""
# pylint: disable=invalid-name, too-many-instance-attributes, too-many-lines

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import StrEnum

from mashumaro.mixins.orjson import DataClassORJSONMixin

from tomtom_apis.api import BaseParams, BasePostData
from tomtom_apis.models import AdrCategoryType, Language, LatitudeLongitude, TravelModeType


class AvoidType(StrEnum):
    """Supported avoid types."""

    TOLL_ROADS = "tollRoads"
    MOTORWAYS = "motorways"
    FERRIES = "ferries"
    UNPAVED_ROADS = "unpavedRoads"
    CARPOOLS = "carpools"
    ALREADY_USED_ROADS = "alreadyUsedRoads"
    BORDER_CROSSINGS = "borderCrossings"
    TUNNELS = "tunnels"
    CAR_TRAINS = "carTrains"
    LOW_EMISSION_ZONES = "lowEmissionZones"


@dataclass(kw_only=True)
class CalculateLongDistanceEVRouteParams(BaseParams):
    """Parameters for the calculate long distance EV route API."""

    vehicleHeading: int | None = None
    sectionType: list[SectionType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    report: str | None = None
    departAt: str | None = None
    routeType: RouteType | None = None
    traffic: bool | None = None
    avoid: list[AvoidType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    travelMode: TravelModeType | None = None
    vehicleMaxSpeed: int | None = None
    vehicleWeight: int | None = None
    vehicleAxleWeight: int | None = None
    vehicleNumberOfAxles: int | None = None
    vehicleLength: float | None = None
    vehicleWidth: float | None = None
    vehicleHeight: float | None = None
    vehicleCommercial: bool | None = None
    vehicleLoadType: list[VehicleLoadType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    vehicleAdrTunnelRestrictionCode: AdrCategoryType | None = None
    vehicleEngineType: VehicleEngineType
    accelerationEfficiency: float | None = None
    decelerationEfficiency: float | None = None
    uphillEfficiency: float | None = None
    downhillEfficiency: float | None = None
    consumptionInkWhPerkmAltitudeGain: float | None = None
    recuperationInkWhPerkmAltitudeLoss: float | None = None
    constantSpeedConsumptionInkWhPerHundredkm: str | None = None
    currentChargeInkWh: float | None = None
    maxChargeInkWh: float | None = None
    auxiliaryPowerInkW: float | None = None
    minChargeAtDestinationInkWh: float | None = None
    minChargeAtChargingStopsInkWh: float | None = None


@dataclass(kw_only=True)
class CalculateLongDistanceEVRoutePostData(BasePostData):
    """Data for the post calculate long distance EV route API."""

    chargingModes: list[ChargingMode]


@dataclass(kw_only=True)
class CalculateReachableRangePostData(BasePostData):
    """Data for the post calculate reachable range API."""

    avoidVignette: list[str] | None = None
    allowVignette: list[str] | None = None
    avoidAreas: Rectangles | None = None


@dataclass(kw_only=True)
class CalculateReachableRouteParams(BaseParams):
    """Parameters for the calculate reachable route API."""

    fuelBudgetInLiters: float | None = None
    energyBudgetInkWh: float | None = None
    timeBudgetInSec: float | None = None
    report: str | None = None
    departAt: str | None = None
    arriveAt: str | None = None
    routeType: RouteType | None = None
    traffic: bool | None = None
    avoid: list[AvoidType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    travelMode: TravelModeType | None = None
    hilliness: HillinessType | None = None
    windingness: WindingnessType | None = None
    vehicleMaxSpeed: int | None = None
    vehicleWeight: int | None = None
    vehicleAxleWeight: int | None = None
    vehicleNumberOfAxles: int | None = None
    vehicleLength: float | None = None
    vehicleWidth: float | None = None
    vehicleHeight: float | None = None
    vehicleCommercial: bool | None = None
    vehicleLoadType: list[VehicleLoadType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    vehicleAdrTunnelRestrictionCode: AdrCategoryType | None = None
    constantSpeedConsumptionInLitersPerHundredkm: str | None = None
    currentFuelInLiters: float | None = None
    auxiliaryPowerInLitersPerHour: float | None = None
    fuelEnergyDensityInMJoulesPerLiter: float | None = None
    accelerationEfficiency: float | None = None
    decelerationEfficiency: float | None = None
    uphillEfficiency: float | None = None
    downhillEfficiency: float | None = None
    consumptionInkWhPerkmAltitudeGain: float | None = None
    recuperationInkWhPerkmAltitudeLoss: float | None = None
    currentChargeInkWh: float | None = None
    maxChargeInkWh: float | None = None
    auxiliaryPowerInkW: float | None = None
    vehicleEngineType: VehicleEngineType | None = None
    constantSpeedConsumptionInkWhPerHundredkm: str | None = None


@dataclass(kw_only=True)
class CalculateRouteParams(BaseParams):
    """Parameters for the calculate route API."""

    maxAlternatives: int | None = None
    instructionsType: InstructionsType | None = None
    language: Language | None = None
    computeBestOrder: bool | None = None
    routeRepresentation: str | None = None
    computeTravelTimeFor: str | None = None
    vehicleHeading: int | None = None
    sectionType: list[SectionType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    report: str | None = None
    departAt: str | None = None
    arriveAt: str | None = None
    routeType: RouteType | None = None
    traffic: bool | None = None
    avoid: list[AvoidType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    travelMode: TravelModeType | None = None
    hilliness: HillinessType | None = None
    windingness: WindingnessType | None = None
    vehicleMaxSpeed: int | None = None  # 0-250
    vehicleWeight: int | None = None
    vehicleAxleWeight: int | None = None
    vehicleNumberOfAxles: int | None = None
    vehicleLength: float | None = None
    vehicleWidth: float | None = None
    vehicleHeight: float | None = None
    vehicleCommercial: bool | None = None
    vehicleLoadType: list[VehicleLoadType] | None = field(default=None, metadata={"serialize": list})  # serialize as list, so the field repeats.
    vehicleAdrTunnelRestrictionCode: AdrCategoryType | None = None
    vehicleEngineType: VehicleEngineType | None = None
    constantSpeedConsumptionInLitersPerHundredkm: str | None = None
    currentFuelInLiters: float | None = None
    auxiliaryPowerInLitersPerHour: float | None = None
    fuelEnergyDensityInMJoulesPerLiter: float | None = None
    accelerationEfficiency: float | None = None
    decelerationEfficiency: float | None = None
    uphillEfficiency: float | None = None
    downhillEfficiency: float | None = None
    consumptionInkWhPerkmAltitudeGain: float | None = None
    recuperationInkWhPerkmAltitudeLoss: float | None = None
    constantSpeedConsumptionInkWhPerHundredkm: str | None = None
    currentChargeInkWh: float | None = None
    maxChargeInkWh: float | None = None
    auxiliaryPowerInkW: float | None = None


@dataclass(kw_only=True)
class CalculateRoutePostData(BasePostData):
    """Data for the post calculate route API."""

    supportingPoints: list[LatitudeLongitude] | None = None
    avoidVignette: list[str] | None = None
    allowVignette: list[str] | None = None
    avoidAreas: Rectangles | None = None


@dataclass(kw_only=True)
class CalculatedLongDistanceEVRouteResponse(DataClassORJSONMixin):
    """Represents a calculated long distance EV route response."""

    formatVersion: str
    routes: list[EVRoute]


@dataclass(kw_only=True)
class CalculatedReachableRangeResponse(DataClassORJSONMixin):
    """Represents a calculated reachable range response."""

    formatVersion: str
    reachableRange: ReachableRange


@dataclass(kw_only=True)
class CalculatedRouteResponse(DataClassORJSONMixin):
    """Represents a calculated route response."""

    formatVersion: str
    routes: list[Route]


@dataclass(kw_only=True)
class ChargingConnection(DataClassORJSONMixin):
    """Represents a charging connection."""

    facilityType: FacilityType
    plugType: PlugType


@dataclass(kw_only=True)
class ChargingConnectionInfo(DataClassORJSONMixin):
    """Represents the charging connection info."""

    chargingVoltageInV: int
    chargingCurrentInA: int
    chargingCurrentType: ChargingCurrentType
    chargingPowerInkW: int
    chargingPlugType: PlugType


class ChargingCurrentType(StrEnum):
    """Supported charging current types."""

    DIRECT_CURRENT = "Direct_Current"
    ALTERNATING_CURRENT_1_PHASE = "Alternating_Current_1_Phase"
    ALTERNATING_CURRENT_3_PHASE = "Alternating_Current_3_Phase"


@dataclass(kw_only=True)
class ChargingCurve(DataClassORJSONMixin):
    """Represents a charging curve."""

    chargeInkWh: float
    timeToChargeInSeconds: int


@dataclass(kw_only=True)
class ChargingInformationAtEndOfLeg(DataClassORJSONMixin):
    """Represents the charging information at the end of a leg."""

    chargingConnections: list[ChargingConnection]
    chargingConnectionInfo: ChargingConnectionInfo
    targetChargeInkWh: float
    chargingTimeInSeconds: int
    chargingParkUuid: str
    chargingParkExternalId: str
    chargingParkName: str
    chargingParkOperatorName: str
    chargingParkLocation: ChargingParkLocation
    chargingParkPaymentOptions: list[ChargingParkPaymentOption]
    chargingParkPowerInkW: int
    chargingStopType: ChargingStopType


@dataclass(kw_only=True)
class ChargingMode(DataClassORJSONMixin):
    """Represents a charging mode."""

    chargingConnections: list[ChargingConnection]
    chargingCurve: list[ChargingCurve]


@dataclass(kw_only=True)
class ChargingParkLocation(DataClassORJSONMixin):
    """Represents a charging park location."""

    coordinate: LatitudeLongitude
    street: str
    city: str
    postalCode: str
    countryCode: str


@dataclass(kw_only=True)
class ChargingParkPaymentOption(DataClassORJSONMixin):
    """Represents a charging park payment option."""

    method: str
    brands: list[str]


class ChargingStopType(StrEnum):
    """Supported charging stop types."""

    AUTO_GENERATED = "Auto_Generated"
    USER_DEFINED = "User_Defined"


@dataclass(kw_only=True)
class EVLeg(DataClassORJSONMixin):
    """Represents a leg of a EV route."""

    summary: EVLegSummary | Summary
    points: list[LatitudeLongitude]


@dataclass(kw_only=True)
class EVLegSummary(DataClassORJSONMixin):
    """Represents the summary of a EV leg."""

    lengthInMeters: int
    travelTimeInSeconds: int
    trafficDelayInSeconds: int
    trafficLengthInMeters: int
    departureTime: datetime
    arrivalTime: datetime
    batteryConsumptionInkWh: float
    remainingChargeAtArrivalInkWh: float
    chargingInformationAtEndOfLeg: ChargingInformationAtEndOfLeg


@dataclass(kw_only=True)
class EVRoute(DataClassORJSONMixin):
    """Represents a EV route."""

    summary: EVSummary
    legs: list[EVLeg]
    sections: list[Section]


@dataclass(kw_only=True)
class EVSummary(DataClassORJSONMixin):
    """Represents the EV summary of a route."""

    lengthInMeters: int
    travelTimeInSeconds: int
    trafficDelayInSeconds: int
    trafficLengthInMeters: int
    departureTime: datetime
    arrivalTime: datetime
    batteryConsumptionInkWh: float
    remainingChargeAtArrivalInkWh: float
    totalChargingTimeInSeconds: float


class FacilityType(StrEnum):
    """Supported facility types."""

    BATTERY_EXCHANGE = "Battery_Exchange"
    CHARGE_100_TO_120V_1_PHASE_AT_8A = "Charge_100_to_120V_1_Phase_at_8A"
    CHARGE_100_TO_120V_1_PHASE_AT_10A = "Charge_100_to_120V_1_Phase_at_10A"
    CHARGE_100_TO_120V_1_PHASE_AT_12A = "Charge_100_to_120V_1_Phase_at_12A"
    CHARGE_100_TO_120V_1_PHASE_AT_13A = "Charge_100_to_120V_1_Phase_at_13A"
    CHARGE_100_TO_120V_1_PHASE_AT_16A = "Charge_100_to_120V_1_Phase_at_16A"
    CHARGE_100_TO_120V_1_PHASE_AT_32A = "Charge_100_to_120V_1_Phase_at_32A"
    CHARGE_200_TO_240V_1_PHASE_AT_8A = "Charge_200_to_240V_1_Phase_at_8A"
    CHARGE_200_TO_240V_1_PHASE_AT_10A = "Charge_200_to_240V_1_Phase_at_10A"
    CHARGE_200_TO_240V_1_PHASE_AT_12A = "Charge_200_to_240V_1_Phase_at_12A"
    CHARGE_200_TO_240V_1_PHASE_AT_16A = "Charge_200_to_240V_1_Phase_at_16A"
    CHARGE_200_TO_240V_1_PHASE_AT_20A = "Charge_200_to_240V_1_Phase_at_20A"
    CHARGE_200_TO_240V_1_PHASE_AT_32A = "Charge_200_to_240V_1_Phase_at_32A"
    CHARGE_200_TO_240V_1_PHASE_ABOVE_32A = "Charge_200_to_240V_1_Phase_above_32A"
    CHARGE_200_TO_240V_3_PHASE_AT_16A = "Charge_200_to_240V_3_Phase_at_16A"
    CHARGE_200_TO_240V_3_PHASE_AT_32A = "Charge_200_to_240V_3_Phase_at_32A"
    CHARGE_380_TO_480V_3_PHASE_AT_16A = "Charge_380_to_480V_3_Phase_at_16A"
    CHARGE_380_TO_480V_3_PHASE_AT_32A = "Charge_380_to_480V_3_Phase_at_32A"
    CHARGE_380_TO_480V_3_PHASE_AT_63A = "Charge_380_to_480V_3_Phase_at_63A"
    CHARGE_50_TO_500V_DIRECT_CURRENT_AT_62A_25KW = "Charge_50_to_500V_Direct_Current_at_62A_25kW"
    CHARGE_50_TO_500V_DIRECT_CURRENT_AT_125A_50KW = "Charge_50_to_500V_Direct_Current_at_125A_50kW"
    CHARGE_200_TO_450V_DIRECT_CURRENT_AT_200A_90KW = "Charge_200_to_450V_Direct_Current_at_200A_90kW"
    CHARGE_200_TO_480V_DIRECT_CURRENT_AT_255A_120KW = "Charge_200_to_480V_Direct_Current_at_255A_120kW"
    CHARGE_DIRECT_CURRENT_AT_20KW = "Charge_Direct_Current_at_20kW"
    CHARGE_DIRECT_CURRENT_AT_50KW = "Charge_Direct_Current_at_50kW"
    CHARGE_DIRECT_CURRENT_ABOVE_50KW = "Charge_Direct_Current_above_50kW"


class HillinessType(StrEnum):
    """Supported hilliness types."""

    LOW = "low"
    HIGH = "high"


class InstructionsType(StrEnum):
    """Supported instructions types."""

    CODED = "coded"
    TEXT = "text"
    TAGGED = "tagged"


@dataclass(kw_only=True)
class Leg(DataClassORJSONMixin):
    """Represents a leg of a route."""

    summary: Summary
    points: list[LatitudeLongitude]


class PlugType(StrEnum):
    """Supported plug types."""

    SMALL_PADDLE_INDUCTIVE = "Small_Paddle_Inductive"
    LARGE_PADDLE_INDUCTIVE = "Large_Paddle_Inductive"
    IEC_60309_1_PHASE = "IEC_60309_1_Phase"
    IEC_60309_3_PHASE = "IEC_60309_3_Phase"
    IEC_62196_TYPE_1_OUTLET = "IEC_62196_Type_1_Outlet"
    IEC_62196_TYPE_2_OUTLET = "IEC_62196_Type_2_Outlet"
    IEC_62196_TYPE_3_OUTLET = "IEC_62196_Type_3_Outlet"
    IEC_62196_TYPE_1_CONNECTOR_CABLE_ATTACHED = "IEC_62196_Type_1_Connector_Cable_Attached"
    IEC_62196_TYPE_2_CONNECTOR_CABLE_ATTACHED = "IEC_62196_Type_2_Connector_Cable_Attached"
    IEC_62196_TYPE_3_CONNECTOR_CABLE_ATTACHED = "IEC_62196_Type_3_Connector_Cable_Attached"
    COMBO_TO_IEC_62196_TYPE_1_BASE = "Combo_to_IEC_62196_Type_1_Base"
    COMBO_TO_IEC_62196_TYPE_2_BASE = "Combo_to_IEC_62196_Type_2_Base"
    TYPE_E_FRENCH_STANDARD_CEE_7_5 = "Type_E_French_Standard_CEE_7_5"
    TYPE_F_SCHUKO_CEE_7_4 = "Type_F_Schuko_CEE_7_4"
    TYPE_G_BRITISH_STANDARD_BS_1363 = "Type_G_British_Standard_BS_1363"
    TYPE_J_SWISS_STANDARD_SEV_1011 = "Type_J_Swiss_Standard_SEV_1011"
    CHINA_GB_PART_2 = "China_GB_Part_2"
    CHINA_GB_PART_3 = "China_GB_Part_3"
    IEC_309_DC_PLUG = "IEC_309_DC_Plug"
    AVCON_CONNECTOR = "AVCON_Connector"
    TESLA_CONNECTOR = "Tesla_Connector"
    NEMA_5_20 = "NEMA_5_20"
    CHADEMO = "CHAdeMO"
    SAE_J1772 = "SAE_J1772"
    TEPCO = "TEPCO"
    BETTER_PLACE_SOCKET = "Better_Place_Socket"
    MARECHAL_SOCKET = "Marechal_Socket"
    STANDARD_HOUSEHOLD_COUNTRY_SPECIFIC = "Standard_Household_Country_Specific"


@dataclass(kw_only=True)
class ReachableRange(DataClassORJSONMixin):
    """Represents a reachable range."""

    center: LatitudeLongitude
    boundary: list[LatitudeLongitude]


@dataclass(kw_only=True)
class Rectangle:
    """A rectangle defined by its south-west and north-east corners."""

    southWestCorner: LatitudeLongitude
    northEastCorner: LatitudeLongitude


@dataclass(kw_only=True)
class Rectangles:
    """A list of rectangles."""

    rectangles: list[Rectangle]


@dataclass(kw_only=True)
class Route(DataClassORJSONMixin):
    """Represents a route."""

    summary: Summary
    legs: list[Leg]
    sections: list[Section]


class RouteType(StrEnum):
    """Supported route types."""

    FASTEST = "fastest"
    SHORTEST = "shortest"
    SHORT = "short"
    ECO = "eco"
    THRILLING = "thrilling"


@dataclass(kw_only=True)
class Section(DataClassORJSONMixin):
    """Represents a section of a route."""

    startPointIndex: int
    endPointIndex: int
    sectionType: SectionReponseType
    travelMode: TravelModeType


class SectionReponseType(StrEnum):
    """Supported section reponse types."""

    CAR_TRAIN = "CAR_TRAIN"
    COUNTRY = "COUNTRY"
    FERRY = "FERRY"
    MOTORWAY = "MOTORWAY"
    PEDESTRIAN = "PEDESTRIAN"
    TOLL = "TOLL"
    TRAFFIC = "TRAFFIC"
    TRAVEL_MODE = "TRAVEL_MODE"
    TUNNEL = "TUNNEL"
    CARPOOL = "CARPOOL"
    URBAN = "URBAN"
    UNPAVED = "UNPAVED"
    LOW_EMISSION_ZONE = "LOW_EMISSION_ZONE"
    SPEED_LIMIT = "SPEED_LIMIT"


class SectionType(StrEnum):
    """Supported section types."""

    CAR_TRAIN = "carTrain"
    FERRY = "ferry"
    TUNNEL = "tunnel"
    MOTORWAY = "motorway"
    PEDESTRIAN = "pedestrian"
    TOLL_ROAD = "tollRoad"
    TOLL_VIGNETTE = "tollVignette"
    COUNTRY = "country"
    TRAVEL_MODE = "travelMode"
    TRAFFIC = "traffic"


@dataclass(kw_only=True)
class Summary(DataClassORJSONMixin):
    """Represents the summary of a route."""

    lengthInMeters: int
    travelTimeInSeconds: int
    trafficDelayInSeconds: int
    trafficLengthInMeters: int
    departureTime: datetime
    arrivalTime: datetime


class VehicleEngineType(StrEnum):
    """Supported vehicle engine types."""

    COMBUSTION = "combustion"
    ELECTRIC = "electric"


class VehicleLoadType(StrEnum):
    """Supported vehicle load types."""

    US_HAZMAT_CLASS_1 = "USHazmatClass1"  # Explosives (USA)
    US_HAZMAT_CLASS_2 = "USHazmatClass2"  # Compressed gas (USA)
    US_HAZMAT_CLASS_3 = "USHazmatClass3"  # Flammable liquids (USA)
    US_HAZMAT_CLASS_4 = "USHazmatClass4"  # Flammable solids (USA)
    US_HAZMAT_CLASS_5 = "USHazmatClass5"  # Oxidizers (USA)
    US_HAZMAT_CLASS_6 = "USHazmatClass6"  # Poisons (USA)
    US_HAZMAT_CLASS_7 = "USHazmatClass7"  # Radioactive (USA)
    US_HAZMAT_CLASS_8 = "USHazmatClass8"  # Corrosives (USA)
    US_HAZMAT_CLASS_9 = "USHazmatClass9"  # Miscellaneous (USA)
    OTHER_HAZMAT_EXPLOSIVE = "otherHazmatExplosive"  # Explosives (Rest of the world)
    OTHER_HAZMAT_GENERAL = "otherHazmatGeneral"  # Miscellaneous (Rest of the world)
    OTHER_HAZMAT_HARMFUL_TO_WATER = "otherHazmatHarmfulToWater"  # Harmful to water (Rest of the world)


@dataclass(kw_only=True)
class WaypointOptimizationOptions:
    """Options for the waypoint optimization API."""

    travelMode: TravelModeType
    vehicleMaxSpeed: int
    vehicleWeight: int
    vehicleAxleWeight: int
    vehicleLength: float
    vehicleWidth: float
    vehicleHeight: float
    vehicleCommercial: bool
    vehicleLoadType: list[VehicleLoadType]
    vehicleAdrTunnelRestrictionCode: AdrCategoryType


@dataclass(kw_only=True)
class WaypointOptimizationPoint:
    """A waypoint optimization point."""

    point: LatitudeLongitude


@dataclass(kw_only=True)
class WaypointOptimizationPostData(BasePostData):
    """Data for the post waypoint optimization API."""

    waypoints: list[WaypointOptimizationPoint]
    options: WaypointOptimizationOptions


@dataclass(kw_only=True)
class WaypointOptimizedResponse(DataClassORJSONMixin):
    """Represents a waypoint optimized response."""

    optimizedOrder: list[int]


class WindingnessType(StrEnum):
    """Supported windingness types."""

    LOW = "low"
    HIGH = "high"



"""MapDisplay tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.maps import MapDisplayApi
from tomtom_apis.maps.models import (
    DangerousGoodsLoadType,
    EmissionClassType,
    EngineType,
    GeneralLoadType,
    IncludeType,
    LayerType,
    LayerTypeWithPoiType,
    MapServiceCopyrightsResponse,
    MapTileV1Params,
    MapTileV2Params,
    StaticImageParams,
    StyleType,
    TileFormatType,
)
from tomtom_apis.models import AdrCategoryType, Language, TravelModeType, ViewType


@pytest.fixture(name="map_display_api")
async def fixture_map_display_api() -> AsyncGenerator[MapDisplayApi]:
    """Fixture for MapDisplayApi."""
    options = ApiOptions(api_key=API_KEY)
    async with MapDisplayApi(options) as map_display:
        yield map_display


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_map_tile.png"], indirect=True)
async def test_deserialization_get_map_tile(map_display_api: MapDisplayApi) -> None:
    """Test the get_map_tile method."""
    response = await map_display_api.get_map_tile(
        layer=LayerType.BASIC,
        style=StyleType.MAIN,
        x=0,
        y=0,
        zoom=0,
        image_format=TileFormatType.PNG,
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_satellite_tile.jpg"], indirect=True)
async def test_deserialization_get_satellite_tile(map_display_api: MapDisplayApi) -> None:
    """Test the get_satellite_tile method."""
    response = await map_display_api.get_satellite_tile(
        x=0,
        y=0,
        zoom=0,
        image_format=TileFormatType.PNG,
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_hillshade_tile.png"], indirect=True)
async def test_deserialization_get_hillshade_tile(map_display_api: MapDisplayApi) -> None:
    """Test the get_hillshade_tile method."""
    response = await map_display_api.get_hillshade_tile(
        x=0,
        y=0,
        zoom=0,
        image_format=TileFormatType.PNG,
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_static_image.png"], indirect=True)
async def test_deserialization_get_static_image(map_display_api: MapDisplayApi) -> None:
    """Test the get_static_image method."""
    response = await map_display_api.get_static_image(
        params=StaticImageParams(
            layer=LayerType.BASIC,
            style=StyleType.MAIN,
            format=TileFormatType.PNG,
            zoom=12,
            center=[4.899886, 52.379031],
            width=512,
            height=512,
            view=ViewType.UNIFIED,
        ),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_tile_v1.pbf"], indirect=True)
async def test_deserialization_get_tile_v1(map_display_api: MapDisplayApi) -> None:
    """Test the get_tile_v1 method."""
    response = await map_display_api.get_tile_v1(
        layer=LayerTypeWithPoiType.BASIC,
        x=0,
        y=0,
        zoom=0,
        params=MapTileV1Params(
            view=ViewType.UNIFIED,
            language=Language.NGT,
        ),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("image_response")
@pytest.mark.parametrize("image_response", ["maps/get_tile_v2.pbf"], indirect=True)
async def test_deserialization_get_tile_v2(map_display_api: MapDisplayApi) -> None:
    """Test the get_tile_v2 method."""
    response = await map_display_api.get_tile_v2(
        layer=LayerTypeWithPoiType.BASIC,
        x=0,
        y=0,
        zoom=0,
        params=MapTileV2Params(
            view=ViewType.UNIFIED,
            include=[IncludeType.ROAD_RESTRICTIONS],
            vehicleWeight=2000,
            vehicleAxleWeight=1000,
            numberOfAxles=2,
            vehicleLength=2.5,
            vehicleWidth=2.5,
            vehicleHeight=2.5,
            generalLoadType=[GeneralLoadType.GENERAL_HAZARDOUS_MATERIALS],
            dangerousGoodsLoadType=[DangerousGoodsLoadType.GASES, DangerousGoodsLoadType.EXPLOSIVES],
            adrCategory=AdrCategoryType.B,
            commercialVehicle=True,
            travelMode=TravelModeType.TAXI,
            emissionClass=[EmissionClassType.EMISSIONCLASS5],
            engineType=[EngineType.DIESEL],
            travelModeProfile="0,1500,,,5.5,,2.2,,,,,,",
        ),
    )

    assert response
    assert isinstance(response, bytes)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["maps/get_map_copyrights.txt"], indirect=True)
async def test_deserialization_get_map_copyrights(map_display_api: MapDisplayApi) -> None:
    """Test the get_map_copyrights method."""
    response = await map_display_api.get_map_copyrights()

    assert response
    assert isinstance(response, str)
    assert "TomTom. All rights reserved." in response


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["maps/get_map_service_copyrights.json"], indirect=True)
async def test_deserialization_get_map_service_copyrights(map_display_api: MapDisplayApi) -> None:
    """Test the get_map_copyrights method."""
    response = await map_display_api.get_map_service_copyrights()

    assert response
    assert isinstance(response, MapServiceCopyrightsResponse)
    assert response.copyrightsCaption == "©TomTom"



"""Fuel prices test."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.automotive import FuelPricesApi
from tomtom_apis.automotive.models import FuelPricesResponse, FuelPrizeParams


@pytest.fixture(name="fuel_prizes_api")
async def fixture_fuel_prizes_api() -> AsyncGenerator[FuelPricesApi]:
    """Fixture for FuelPricesApi."""
    options = ApiOptions(api_key=API_KEY)
    async with FuelPricesApi(options) as fuel_prizes:
        yield fuel_prizes


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["automotive/get_fuel_prize.json"], indirect=True)
async def test_deserialization_get_fuel_prize(fuel_prizes_api: FuelPricesApi) -> None:
    """Test the get_fuel_prize method."""
    response = await fuel_prizes_api.get_fuel_prize(params=FuelPrizeParams(fuelPrice="1:2622f89a-6300-11ec-8d12-a0423f39b5a2"))

    assert response
    assert isinstance(response, FuelPricesResponse)
    assert response.fuels
    assert response.fuels[0]
    assert response.fuels[0].updatedAt.isoformat() == "2021-12-12T11:29:14+00:00"



"""Models for the TomTom Maps API."""
# pylint: disable=invalid-name, too-many-instance-attributes, too-many-lines

from __future__ import annotations

from dataclasses import dataclass, field
from enum import StrEnum

from mashumaro.mixins.orjson import DataClassORJSONMixin

from tomtom_apis.api import BaseParams
from tomtom_apis.models import AdrCategoryType, Language, TileSizeType, TravelModeType, ViewType
from tomtom_apis.utils import serialize_list_brackets


class DangerousGoodsLoadType(StrEnum):
    """Supported dangerous good types."""

    EXPLOSIVES = "Explosives"
    GASES = "Gases"
    FLAMMABLE_LIQUIDS = "Flammable_Liquids"
    FLAMMABLE_SOLIDS = "Flammable_Solids"
    OXIDIZING_AND_ORGANIC_SUBSTANCE = "Oxidizing_And_Organic_Substance"
    TOXIC_AND_INFECTIOUS_SUBSTANCE = "Toxic_And_Infectious_Substance"
    RADIOACTIVE_MATERIAL = "Radioactive_Material"
    CORROSIVES = "Corrosives"
    MISCELLANEOUS_DANGEROUS_GOODS = "Miscellaneous_Dangerous_Goods"


class EmissionClassType(StrEnum):
    """Supported emission class types."""

    EMISSIONCLASS0 = "EmissionClass0"
    EMISSIONCLASS1 = "EmissionClass1"
    EMISSIONCLASS2 = "EmissionClass2"
    EMISSIONCLASS3 = "EmissionClass3"
    EMISSIONCLASS4 = "EmissionClass4"
    EMISSIONCLASS5 = "EmissionClass5"
    EMISSIONCLASS6 = "EmissionClass6"
    EMISSIONCLASS7 = "EmissionClass7"
    EMISSIONCLASS8 = "EmissionClass8"
    EMISSIONCLASS9 = "EmissionClass9"
    EMISSIONCLASS10 = "EmissionClass10"


class EngineType(StrEnum):
    """Supported engine types."""

    LPG = "LPG"
    CNG = "CNG"
    LNG = "LNG"
    DIESEL = "Diesel"
    PETROL = "Petrol"
    HYDROGEN = "Hydrogen"
    ELECTRIC = "Electric"
    HYBRID = "Hybrid"
    PLUGIN_HYBRID = "Plugin_Hybrid"


class GeneralLoadType(StrEnum):
    """Supported general load types."""

    GENERAL_HAZARDOUS_MATERIALS = "General_Hazardous_Materials"
    EXPLOSIVE_MATERIALS = "Explosive_Materials"
    GOODS_HARMFUL_TO_WATER = "Goods_Harmful_To_Water"


class IncludeType(StrEnum):
    """Supported include types."""

    ROAD_RESTRICTIONS = "road_restrictions"


class LayerType(StrEnum):
    """Supported layer types."""

    BASIC = "basic"
    HYBRID = "hybrid"
    LABELS = "labels"


class LayerTypeWithPoiType(StrEnum):
    """Supported layer types."""

    BASIC = "basic"
    HYBRID = "hybrid"
    LABELS = "labels"
    POI = "poi"


@dataclass(kw_only=True)
class MapServiceCopyrightsResponse(DataClassORJSONMixin):
    """Represents the map service copyrights response."""

    formatVersion: str
    copyrightsCaption: str


@dataclass(kw_only=True)
class MapTileParams(BaseParams):
    """Parameters for the map tile API."""

    tileSize: TileSizeType | None = None
    view: ViewType | None = None
    language: Language | None = None


@dataclass(kw_only=True)
class MapTileV1Params(BaseParams):
    """Parameters for the map tile API."""

    view: ViewType | None = None
    language: Language | None = None


@dataclass(kw_only=True)
class MapTileV2Params(BaseParams):
    """Parameters for the map tile API."""

    view: ViewType | None = None
    include: list[IncludeType] | None = field(default=None, metadata={"serialize": serialize_list_brackets})
    vehicleWeight: int | None = None
    vehicleAxleWeight: int | None = None
    numberOfAxles: int | None = None
    vehicleLength: float | None = None
    vehicleWidth: float | None = None
    vehicleHeight: float | None = None
    generalLoadType: list[GeneralLoadType] | None = field(default=None, metadata={"serialize": serialize_list_brackets})
    dangerousGoodsLoadType: list[DangerousGoodsLoadType] | None = field(default=None, metadata={"serialize": serialize_list_brackets})
    adrCategory: AdrCategoryType | None = None
    commercialVehicle: bool | None = None
    travelMode: TravelModeType | None = None
    emissionClass: list[EmissionClassType] | None = field(default=None, metadata={"serialize": serialize_list_brackets})
    engineType: list[EngineType] | None = field(default=None, metadata={"serialize": serialize_list_brackets})
    travelModeProfile: str | None = None


@dataclass(kw_only=True)
class StaticImageParams(BaseParams):
    """Parameters for the map tile API."""

    layer: LayerType | None = None
    style: StyleType | None = None
    x: int | None = None
    y: int | None = None
    zoom: int | None = None
    center: list[float] | None = None
    format: TileFormatType | None = None
    width: int | None = None  # must be a positive integer between 1 and 8192.
    height: int | None = None  # must be a positive integer between 1 and 8192.
    bbox: list[float] | None = None
    view: ViewType | None = None


class StyleType(StrEnum):
    """Supported style types."""

    MAIN = "main"
    NIGHT = "night"


class TileFormatType(StrEnum):
    """Supported tile formats."""

    PNG = "png"
    JPG = "jpg"



"""Routing tests."""

from collections.abc import AsyncGenerator
from datetime import datetime

import pytest

from tests.const import API_KEY, LOC_AMSTERDAM, LOC_ROTTERDAM
from tomtom_apis.api import ApiOptions
from tomtom_apis.models import LatLonList, TravelModeType
from tomtom_apis.routing import RoutingApi
from tomtom_apis.routing.models import (
    AvoidType,
    CalculatedReachableRangeResponse,
    CalculatedRouteResponse,
    CalculateReachableRangePostData,
    CalculateReachableRouteParams,
    CalculateRouteParams,
    CalculateRoutePostData,
    RouteType,
    VehicleEngineType,
)


@pytest.fixture(name="routing_api")
async def fixture_routing_api() -> AsyncGenerator[RoutingApi]:
    """Fixture for RoutingApi."""
    options = ApiOptions(api_key=API_KEY)
    async with RoutingApi(options) as routing:
        yield routing


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/routing/get_calculate_route.json"], indirect=True)
async def test_deserialization_get_calculate_route(routing_api: RoutingApi) -> None:
    """Test the get_calculate_route method."""
    response = await routing_api.get_calculate_route(
        locations=LatLonList(locations=[LOC_AMSTERDAM, LOC_ROTTERDAM]),
        params=CalculateRouteParams(
            maxAlternatives=0,
            routeType=RouteType.FASTEST,
            traffic=True,
            travelMode=TravelModeType.CAR,
        ),
    )

    assert response
    assert isinstance(response, CalculatedRouteResponse)
    assert len(response.routes) == 1
    assert response.routes[0].summary
    assert response.routes[0].summary.lengthInMeters > 1000
    assert response.routes[0].summary.travelTimeInSeconds > 30
    assert isinstance(response.routes[0].summary.departureTime, datetime)
    assert isinstance(response.routes[0].summary.arrivalTime, datetime)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/routing/post_calculate_route.json"], indirect=True)
async def test_deserialization_post_calculate_route(routing_api: RoutingApi) -> None:
    """Test the post_calculate_route method."""
    response = await routing_api.post_calculate_route(
        locations=LatLonList(locations=[LOC_AMSTERDAM, LOC_ROTTERDAM]),
        params=CalculateRouteParams(
            maxAlternatives=0,
            routeType=RouteType.FASTEST,
            traffic=True,
            travelMode=TravelModeType.CAR,
        ),
        data=CalculateRoutePostData.from_dict(
            {
                "supportingPoints": [{"latitude": 52.5093, "longitude": 13.42936}, {"latitude": 52.50844, "longitude": 13.42859}],
                "avoidVignette": ["AUS", "CHE"],
                "avoidAreas": {
                    "rectangles": [
                        {
                            "southWestCorner": {"latitude": 48.81851, "longitude": 2.26593},
                            "northEastCorner": {"latitude": 48.90309, "longitude": 2.41115},
                        },
                    ],
                },
            },
        ),
    )

    assert response
    assert isinstance(response, CalculatedRouteResponse)
    assert len(response.routes) == 1
    assert response.routes[0].summary
    assert response.routes[0].summary.lengthInMeters > 1000
    assert response.routes[0].summary.travelTimeInSeconds > 30
    assert isinstance(response.routes[0].summary.departureTime, datetime)
    assert isinstance(response.routes[0].summary.arrivalTime, datetime)
    assert isinstance(response.routes[0].legs[0].summary.departureTime, datetime)
    assert isinstance(response.routes[0].legs[0].summary.arrivalTime, datetime)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/routing/get_calculate_reachable_range.json"], indirect=True)
async def test_deserialization_get_calculate_reachable_range(routing_api: RoutingApi) -> None:
    """Test the get_calculate_reachable_range method."""
    response = await routing_api.get_calculate_reachable_range(
        origin=LOC_AMSTERDAM,
        params=CalculateReachableRouteParams(
            energyBudgetInkWh=43,
            avoid=[AvoidType.UNPAVED_ROADS],
            vehicleEngineType=VehicleEngineType.ELECTRIC,
            constantSpeedConsumptionInkWhPerHundredkm="50,8.2:130,21.3",
        ),
    )

    assert response
    assert isinstance(response, CalculatedReachableRangeResponse)
    assert response.reachableRange
    assert response.reachableRange.center
    assert response.reachableRange.center.latitude == 52.50931
    assert response.reachableRange.center.longitude == 13.42937
    assert len(response.reachableRange.boundary) > 1


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["routing/routing/post_calculate_reachable_range.json"], indirect=True)
async def test_deserialization_post_calculate_reachable_range(routing_api: RoutingApi) -> None:
    """Test the post_calculate_reachable_range method."""
    response = await routing_api.post_calculate_reachable_range(
        origin=LOC_AMSTERDAM,
        params=CalculateReachableRouteParams(
            energyBudgetInkWh=43,
            avoid=[AvoidType.UNPAVED_ROADS],
            vehicleEngineType=VehicleEngineType.ELECTRIC,
            constantSpeedConsumptionInkWhPerHundredkm="50,8.2:130,21.3",
        ),
        data=CalculateReachableRangePostData.from_dict(
            {
                "avoidVignette": ["AUS", "CHE"],
                "avoidAreas": {
                    "rectangles": [
                        {
                            "southWestCorner": {"latitude": 48.81851, "longitude": 2.26593},
                            "northEastCorner": {"latitude": 48.90309, "longitude": 2.41115},
                        },
                    ],
                },
            },
        ),
    )

    assert response
    assert isinstance(response, CalculatedReachableRangeResponse)
    assert response.reachableRange
    assert response.reachableRange.center
    assert response.reachableRange.center.latitude == 52.50931
    assert response.reachableRange.center.longitude == 13.42937
    assert len(response.reachableRange.boundary) > 1



"""Utility to sort classes in a file."""

import re
import subprocess
import sys
from pathlib import Path

PROJECT_ROOT = Path(__file__).parent.parent


def sort_classes_alphabetically(file_path: Path) -> None:
    """Sort classes alphabetically in a given file."""
    with file_path.open(encoding="UTF-8") as file:
        content = file.read()

    # Regular expression to match class definitions, including decorators and docstrings.
    class_pattern = re.compile(
        r"(@[^\n]+\n)*"  # Match decorators
        r"(class\s+\w+.*?:\n)"  # Match class definition line
        r"((?:\s{4}.*\n)*)",  # Match class content (indented lines)
    )

    # Find all classes.
    classes: list[tuple[str, str, str]] = class_pattern.findall(content)

    # Sort classes alphabetically by their class names.
    def get_class_name(class_match: tuple[str, str, str]) -> str:
        match = re.search(r"class\s+(\w+)", class_match[1])
        if match:
            return match.group(1)
        return ""  # Fallback in case no match is found, though it shouldn't happen here.

    sorted_classes = sorted(classes, key=get_class_name)

    # Reconstruct the file content with sorted classes.
    sorted_classes_content = ["".join(cls) for cls in sorted_classes]
    non_class_content = class_pattern.sub("", content).strip()

    new_content = non_class_content + "\n\n\n" + "\n\n".join(sorted_classes_content)

    # Write the new content back to the file.
    with file_path.open(mode="w", encoding="UTF-8") as file:
        file.write(new_content)


if __name__ == "__main__":
    if len(sys.argv) == 1:
        print(f"Usage: python {sys.argv[0]} <file_path>")
        sys.exit(1)

    file_to_sort = Path(sys.argv[1])
    if not file_to_sort.exists():
        print(f"File '{file_to_sort}' does not exist")
        sys.exit(1)

    sort_classes_alphabetically(file_to_sort)
    print(f"Classes in '{file_to_sort}' have been sorted alphabetically")

    # Format the file.
    subprocess.run(
        [f"{PROJECT_ROOT}/.venv/bin/ruff", "format", file_to_sort],
        check=True,
        stdout=subprocess.DEVNULL,
    )



"""Matrix Routing V2 tests."""

import pytest

from tomtom_apis.routing import MatrixRoutingApiV2


def test_api_not_implemented() -> None:
    """Test API not implemented yet."""
    with pytest.raises(NotImplementedError):
        MatrixRoutingApiV2()



"""Routing tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.models import Language, LatLon, ViewType
from tomtom_apis.places import SearchApi
from tomtom_apis.places.models import (
    AdditionalDataResponse,
    Address,
    AutocompleteResponse,
    CategorySearchParams,
    Geometry,
    GeometryFilterData,
    GeometryFilterResponse,
    GeometryPoi,
    GeometrySearchParams,
    GeometrySearchPostData,
    NearbySearchParams,
    PlaceByIdParams,
    Poi,
    PoiCategoriesParams,
    PoiCategoriesResponse,
    Points,
    PoiSearchParams,
    RelatedPoisType,
    SearchAlongRouteData,
    SearchAlongRouteParams,
    SearchParams,
    SearchResponse,
    SortByType,
)


@pytest.fixture(name="search_api")
async def fixture_search_api() -> AsyncGenerator[SearchApi]:
    """Fixture for SearchApi."""
    options = ApiOptions(api_key=API_KEY)
    async with SearchApi(options) as search:
        yield search


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_search.json"], indirect=True)
async def test_deserialization_get_search(search_api: SearchApi) -> None:
    """Test the get_search method."""
    response = await search_api.get_search(
        query="pizza",
        params=SearchParams(
            lat=37.337,
            lon=-121.89,
            categorySet=["7315"],
            view=ViewType.UNIFIED,
            relatedPois=RelatedPoisType.OFF,
            minFuzzyLevel=1,
            maxFuzzyLevel=2,
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_poi_search.json"], indirect=True)
async def test_deserialization_get_poi_search(search_api: SearchApi) -> None:
    """Test the get_poi_search method."""
    response = await search_api.get_poi_search(
        query="pizza",
        params=PoiSearchParams(
            lat=37.337,
            lon=-121.89,
            view=ViewType.UNIFIED,
            relatedPois=RelatedPoisType.OFF,
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_category_search.json"], indirect=True)
async def test_deserialization_get_category_search(search_api: SearchApi) -> None:
    """Test the get_category_search method."""
    response = await search_api.get_category_search(
        query="pizza",
        params=CategorySearchParams(
            lat=37.337,
            lon=-121.89,
            categorySet=["7315"],
            view=ViewType.UNIFIED,
            relatedPois=RelatedPoisType.OFF,
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_geometry_search.json"], indirect=True)
async def test_deserialization_get_geometry_search(search_api: SearchApi) -> None:
    """Test the get_geometry_search method."""
    response = await search_api.get_geometry_search(
        query="pizza",
        geometryList=[
            Geometry(
                type="POLYGON",
                vertices=[
                    "37.7524152343544, -122.43576049804686",
                    "37.70660472542312, -122.43301391601562",
                    "37.712059855877314, -122.36434936523438",
                    "37.75350561243041, -122.37396240234374",
                ],
            ),
            Geometry(
                type="CIRCLE",
                position="37.71205, -121.36434",
                radius=6000,
            ),
            Geometry(
                type="CIRCLE",
                position="37.31205, -121.36434",
                radius=1000,
            ),
        ],
        params=GeometrySearchParams(
            categorySet=["7315"],
            view=ViewType.UNIFIED,
            relatedPois=RelatedPoisType.OFF,
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/post_geometry_search.json"], indirect=True)
async def test_deserialization_post_geometry_search(search_api: SearchApi) -> None:
    """Test the post_geometry_search method."""
    response = await search_api.post_geometry_search(
        query="pizza",
        params=GeometrySearchParams(
            categorySet=["7315"],
            view=ViewType.UNIFIED,
            relatedPois=RelatedPoisType.OFF,
        ),
        data=GeometrySearchPostData(
            geometryList=[
                Geometry(
                    type="POLYGON",
                    vertices=[
                        "37.7524152343544, -122.43576049804686",
                        "37.70660472542312, -122.43301391601562",
                        "37.712059855877314, -122.36434936523438",
                        "37.75350561243041, -122.37396240234374",
                    ],
                ),
                Geometry(
                    type="CIRCLE",
                    position="37.71205, -121.36434",
                    radius=6000,
                ),
                Geometry(
                    type="CIRCLE",
                    position="37.31205, -121.36434",
                    radius=1000,
                ),
            ],
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_nearby_search.json"], indirect=True)
async def test_deserialization_get_nearby_search(search_api: SearchApi) -> None:
    """Test the get_nearby_search method."""
    response = await search_api.get_nearby_search(
        lat=48.872263,
        lon=2.299541,
        params=NearbySearchParams(radius=1000, limit=100),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 5


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/post_search_along_route.json"], indirect=True)
async def test_deserialization_post_search_along_route(search_api: SearchApi) -> None:
    """Test the post_search_along_route method."""
    response = await search_api.post_search_along_route(
        query="pizza",
        maxDetourTime=600,
        params=SearchAlongRouteParams(
            categorySet=["7315"],
            view=ViewType.UNIFIED,
            sortBy=SortByType.DETOUR_OFFSET,
            relatedPois=RelatedPoisType.OFF,
        ),
        data=SearchAlongRouteData(
            route=Points(
                points=[
                    LatLon(lat=37.52768, lon=-122.30082),
                    LatLon(lat=37.52952, lon=-122.29365),
                    LatLon(lat=37.52987, lon=-122.2883),
                    LatLon(lat=37.52561, lon=-122.28219),
                    LatLon(lat=37.52091, lon=-122.27661),
                    LatLon(lat=37.52277, lon=-122.27334),
                    LatLon(lat=37.52432, lon=-122.26833),
                    LatLon(lat=37.5139, lon=-122.25621),
                    LatLon(lat=37.49881, lon=-122.2391),
                    LatLon(lat=37.49426, lon=-122.2262),
                    LatLon(lat=37.48792, lon=-122.20905),
                    LatLon(lat=37.48425, lon=-122.18374),
                    LatLon(lat=37.47642, lon=-122.1683),
                    LatLon(lat=37.4686, lon=-122.15644),
                    LatLon(lat=37.46981, lon=-122.15498),
                    LatLon(lat=37.4718, lon=-122.15149),
                ],
            ),
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert response.results
    assert len(response.results) > 3


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_place_by_id.json"], indirect=True)
async def test_deserialization_get_place_by_id(search_api: SearchApi) -> None:
    """Test the test_get_place_by_id method."""
    response = await search_api.get_place_by_id(
        params=PlaceByIdParams(entityId="528009004256119"),
    )

    assert response
    assert response.results
    assert len(response.results) == 1
    assert response.results[0]
    assert response.results[0].poi
    assert response.results[0].poi.name == "Amsterdam Central"


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_autocomplete.json"], indirect=True)
async def test_deserialization_get_autocomplete(search_api: SearchApi) -> None:
    """Test the get_autocomplete method."""
    response = await search_api.get_autocomplete(query="pizza", language=Language.EN_US)

    assert response
    assert isinstance(response, AutocompleteResponse)
    assert response.results
    assert len(response.results) > 2


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_geometry_filter.json"], indirect=True)
async def test_deserialization_get_geometry_filter(search_api: SearchApi) -> None:
    """Test the get_geometry_filter method."""
    response = await search_api.get_geometry_filter(
        geometryList=[
            Geometry(type="CIRCLE", position="40.80558, -73.96548", radius=100),
            Geometry(
                type="POLYGON",
                vertices=[
                    "37.7524152343544, -122.43576049804686",
                    "37.70660472542312, -122.43301391601562",
                    "37.712059855877314, -122.36434936523438",
                    "37.75350561243041, -122.37396240234374",
                ],
            ),
        ],
        poiList=[
            GeometryPoi(
                poi=Poi(name="S Restaurant Toms"),
                address=Address(freeformAddress="2880 Broadway, New York, NY 10025"),
                position=LatLon(lat=40.80558, lon=-73.96548),
            ),
            GeometryPoi(
                poi=Poi(name="Yasha Raman Corporation"),
                address=Address(freeformAddress="940 Amsterdam Ave, New York, NY 10025"),
                position=LatLon(lat=40.80076, lon=-73.96556),
            ),
        ],
    )

    assert response
    assert isinstance(response, GeometryFilterResponse)
    assert response.results
    assert len(response.results) > 0


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/post_geometry_filter.json"], indirect=True)
async def test_deserialization_post_geometry_filter(search_api: SearchApi) -> None:
    """Test the post_geometry_filter method."""
    response = await search_api.post_geometry_filter(
        data=GeometryFilterData(
            geometryList=[
                Geometry(type="CIRCLE", position="40.80558, -73.96548", radius=100),
                Geometry(
                    type="POLYGON",
                    vertices=[
                        "37.7524152343544, -122.43576049804686",
                        "37.70660472542312, -122.43301391601562",
                        "37.712059855877314, -122.36434936523438",
                        "37.75350561243041, -122.37396240234374",
                    ],
                ),
            ],
            poiList=[
                GeometryPoi(
                    poi=Poi(name="S Restaurant Toms"),
                    address=Address(freeformAddress="2880 Broadway, New York, NY 10025"),
                    position=LatLon(lat=40.80558, lon=-73.96548),
                ),
                GeometryPoi(
                    poi=Poi(name="Yasha Raman Corporation"),
                    address=Address(freeformAddress="940 Amsterdam Ave, New York, NY 10025"),
                    position=LatLon(lat=40.80076, lon=-73.96556),
                ),
            ],
        ),
    )

    assert response
    assert isinstance(response, GeometryFilterResponse)
    assert response.results
    assert len(response.results) > 0


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_poi_categories.json"], indirect=True)
async def test_deserialization_get_poi_categories(search_api: SearchApi) -> None:
    """Test the get_poi_categories method."""
    response = await search_api.get_poi_categories(
        params=PoiCategoriesParams(),
    )

    assert response
    assert isinstance(response, PoiCategoriesResponse)
    assert response.poiCategories
    assert len(response.poiCategories) > 500


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/search/get_additional_data.json"], indirect=True)
async def test_deserialization_get_additional_data(search_api: SearchApi) -> None:
    """Test the get_additional_data method."""
    response = await search_api.get_additional_data(
        geometries=["00004631-3400-3c00-0000-0000673c4d2e", "00004631-3400-3c00-0000-0000673c42fe"],
    )

    assert response
    assert isinstance(response, AdditionalDataResponse)



"""Constants for the TomTom API."""

from enum import IntEnum, StrEnum
from typing import Final

TRACKING_ID_HEADER: Final[str] = "Tracking-ID"
TOMTOM_HEADER_PREFIX: Final[str] = "x-tomtom"


class HttpMethod(StrEnum):
    """HTTP methods used in TomTom API requests."""

    DELETE = "DELETE"
    GET = "GET"
    POST = "POST"
    PUT = "PUT"


class HttpStatus(IntEnum):
    """HTTP status codes used in TomTom API responses."""

    OK = 200
    UNASSIGNED = 399
    BAD_REQUEST = 400
    INTERNAL_SERVER_ERROR = 500



"""Monaco GP routing example."""

# pylint: disable=duplicate-code

import asyncio
import os

from tomtom_apis import ApiOptions
from tomtom_apis.models import LatLon, LatLonList, TravelModeType
from tomtom_apis.routing import RoutingApi
from tomtom_apis.routing.models import CalculateRouteParams, RouteType


async def plan_monaco_gp_route(api_key: str) -> None:
    """Plan Monaco GP route."""
    options = ApiOptions(api_key=api_key)
    async with RoutingApi(options) as routing_api:
        response = await routing_api.get_calculate_route(
            locations=LatLonList(
                locations=[
                    LatLon(lat=43.735053716368, lon=7.4212753772736),
                    LatLon(lat=43.736654495285, lon=7.4214953184128),
                    LatLon(lat=43.737361847639, lon=7.4241694808006),
                    LatLon(lat=43.737974233613, lon=7.4272459745407),
                    LatLon(lat=43.739218365576, lon=7.4273908138275),
                    LatLon(lat=43.740999250921, lon=7.4286353588104),
                    LatLon(lat=43.740235743995, lon=7.4295258522034),
                    LatLon(lat=43.740819033159, lon=7.4293032288551),
                    LatLon(lat=43.741088390688, lon=7.4302071332932),
                    LatLon(lat=43.740410149989, lon=7.4303358793259),
                    LatLon(lat=43.737226191671, lon=7.4255266785622),
                    LatLon(lat=43.736790152549, lon=7.4223375320435),
                    LatLon(lat=43.735567288116, lon=7.4218171834946),
                    LatLon(lat=43.73259432495, lon=7.4234801530838),
                    LatLon(lat=43.732390824682, lon=7.4230295419693),
                    LatLon(lat=43.73454595431, lon=7.4213665723801),
                ],
            ),
            params=CalculateRouteParams(
                maxAlternatives=0,
                routeType=RouteType.FASTEST,
                travelMode=TravelModeType.CAR,
            ),
        )

        # Monaco GP route statistics, source: https://en.wikipedia.org/wiki/Circuit_de_Monaco
        official_track_length = 3337
        lap_record_time = "1:12.909"
        lap_record_seconds = 73
        public_route_length = response.routes[0].summary.lengthInMeters
        driving_time_seconds = response.routes[0].summary.travelTimeInSeconds

        # Format driving time into minutes and seconds
        driving_time_minutes = driving_time_seconds // 60
        driving_time_remaining_seconds = driving_time_seconds % 60

        # Enhanced output
        print("=== Monaco Grand Prix Route Stats ===\n")
        print(f"🏁 Official F1 Track Length:  {official_track_length:,}".replace(",", ".") + " meters")
        print(f"🚗 Street Driving Length:     {public_route_length:,}".replace(",", ".") + " meters")
        print(
            f"⏱️  Estimated Driving Time:    {int(driving_time_minutes)} minutes {int(driving_time_remaining_seconds)}"
            f" seconds ({driving_time_seconds} seconds)",
        )
        print(f"🏎️  F1 Lap Record:             {lap_record_time} ({lap_record_seconds} seconds)\n")
        print("Can you beat the record? Give it a try and see how close you can get!")


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()
    asyncio.run(plan_monaco_gp_route(api_key=user_api_key))



"""Geocoding tests."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.places import GeocodingApi
from tomtom_apis.places.models import ResultType, SearchResponse, StructuredGeocodeParams


@pytest.fixture(name="geocoding_api")
async def fixture_geocoding_api() -> AsyncGenerator[GeocodingApi]:
    """Fixture for GeocodingApi."""
    options = ApiOptions(api_key=API_KEY)
    async with GeocodingApi(options) as geocoding:
        yield geocoding


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/geocoding/get_geocode.json"], indirect=True)
async def test_deserialization_get_geocode(geocoding_api: GeocodingApi) -> None:
    """Test the get_geocode method."""
    response = await geocoding_api.get_geocode(
        query="De Ruijterkade 154 Amsterdam",
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert len(response.results) == 1
    assert response.results[0]
    assert response.results[0].type
    assert response.results[0].type == ResultType.POINT_ADDRESS
    assert response.results[0].position
    assert round(response.results[0].position.lat, 5) == 52.37727
    assert round(response.results[0].position.lon, 5) == 4.90943


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/geocoding/get_structured_geocode.json"], indirect=True)
async def test_deserialization_get_structured_geocode(geocoding_api: GeocodingApi) -> None:
    """Test the get_structured_geocode method."""
    response = await geocoding_api.get_structured_geocode(
        countryCode="NL",
        params=StructuredGeocodeParams(
            streetName="De Ruijterkade",
            streetNumber="154",
            postalCode="1011 AC",
            municipality="Amsterdam",
        ),
    )

    assert response
    assert isinstance(response, SearchResponse)
    assert len(response.results) == 1
    assert response.results[0]
    assert response.results[0].type
    assert response.results[0].type == ResultType.POINT_ADDRESS
    assert response.results[0].position
    assert round(response.results[0].position.lat, 5) == 52.37727
    assert round(response.results[0].position.lon, 5) == 4.90943



"""London traffic example."""

# pylint: disable=duplicate-code

import asyncio
import os
from pathlib import Path

from tomtom_apis import ApiOptions
from tomtom_apis.maps import MapDisplayApi
from tomtom_apis.maps.models import LayerType, StyleType, TileFormatType
from tomtom_apis.models import MapTile
from tomtom_apis.traffic import TrafficApi
from tomtom_apis.traffic.models import IncidentStyleType, IncidentTileFormatType

SCRIPT_DIR = Path(__file__).parent
TILES: list[MapTile] = [  # a 3x3 grid of London at zoom level 10.
    MapTile(x=510, y=339, zoom=10),
    MapTile(x=511, y=339, zoom=10),
    MapTile(x=512, y=339, zoom=10),
    MapTile(x=510, y=340, zoom=10),
    MapTile(x=511, y=340, zoom=10),
    MapTile(x=512, y=340, zoom=10),
    MapTile(x=510, y=341, zoom=10),
    MapTile(x=511, y=341, zoom=10),
    MapTile(x=512, y=341, zoom=10),
]


class TileTypeExceptionError(Exception):
    """Exception raised when the tile type is not as expected."""

    def __init__(self) -> None:
        """Initialize the TileTypeException."""
        super().__init__("Invalid tile type provided.")


async def download_tiles(api: MapDisplayApi | TrafficApi, tiles: list[MapTile]) -> None:
    """Download tiles for a given api.

    Args:
        api (MapDisplayApi | TrafficApi): The API to use for downloading tiles.
        tiles (list[MapTile]): The tiles to download.

    """
    for tile in tiles:
        if isinstance(api, MapDisplayApi):
            image_bytes = await api.get_map_tile(
                layer=LayerType.BASIC,
                style=StyleType.MAIN,
                x=tile.x,
                y=tile.y,
                zoom=tile.zoom,
                image_format=TileFormatType.PNG,
            )
            file_path = SCRIPT_DIR / "tiles" / f"main_{tile.zoom}_{tile.x}_{tile.y}.png"
        elif isinstance(api, TrafficApi):
            image_bytes = await api.get_raster_incident_tile(
                style=IncidentStyleType.S1,
                x=tile.x,
                y=tile.y,
                zoom=tile.zoom,
                image_format=IncidentTileFormatType.PNG,
            )
            file_path = SCRIPT_DIR / "tiles" / f"incidents_{tile.zoom}_{tile.x}_{tile.y}.png"
        else:
            raise TileTypeExceptionError

        with file_path.open("wb") as file:
            file.write(image_bytes)


async def download(api_key: str) -> None:
    """Download all tiles."""
    options = ApiOptions(api_key=api_key)

    async with MapDisplayApi(options) as map_display_api, TrafficApi(options) as traffic_api:
        await download_tiles(map_display_api, TILES)
        await download_tiles(traffic_api, TILES)


def get_api_key() -> str:
    """Get the API key or ask for user input."""
    apik_key = os.getenv("TOMTOM_API_KEY")

    if apik_key:
        return apik_key

    return input("Please enter your API key: ")


if __name__ == "__main__":
    user_api_key = get_api_key()
    asyncio.run(download(user_api_key))



"""Parking Availability test."""

from collections.abc import AsyncGenerator

import pytest

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.automotive import ParkingAvailabilityApi
from tomtom_apis.automotive.models import ParkingAvailabilityParams, ParkingAvailabilityResponse


@pytest.fixture(name="parking_availability_api")
async def fixture_parking_availability_api() -> AsyncGenerator[ParkingAvailabilityApi]:
    """Fixture for ParkingAvailabilityApi."""
    options = ApiOptions(api_key=API_KEY)
    async with ParkingAvailabilityApi(options) as fuel_prizes:
        yield fuel_prizes


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["automotive/get_parking_availability.json"], indirect=True)
async def test_deserialization_get_parking_availability(parking_availability_api: ParkingAvailabilityApi) -> None:
    """Test the get_fuel_prize method."""
    response = await parking_availability_api.get_parking_availability(
        params=ParkingAvailabilityParams(parkingAvailability="00000000-0003-1d9a-0009-20d4467654e2"),
    )

    assert response
    assert isinstance(response, ParkingAvailabilityResponse)
    assert response.statuses
    assert response.statuses[0]
    assert response.statuses[0].current
    assert response.statuses[0].current.updatedAt.isoformat() == "2021-12-12T11:29:00+00:00"



"""EV Search test."""

from collections.abc import AsyncGenerator

import pytest
from aresponses import ResponsesMockServer

from tests.const import API_KEY
from tomtom_apis.api import ApiOptions
from tomtom_apis.models import LatLon
from tomtom_apis.places import BatchSearchApi
from tomtom_apis.places.models import AsynchronousBatchDownloadParams, BatchItem, BatchPostData, BatchResponse, Geometry, Route


@pytest.fixture(name="batch_search_api")
async def fixture_batch_search_api() -> AsyncGenerator[BatchSearchApi]:
    """Fixture for BatchSearchApi."""
    options = ApiOptions(api_key=API_KEY)
    async with BatchSearchApi(options) as batch_search:
        yield batch_search


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/batch_search/post_synchronous_batch.json"], indirect=True)
async def test_deserialization_post_synchronous_batch(batch_search_api: BatchSearchApi) -> None:
    """Test the post_synchronous_batch method."""
    response = await batch_search_api.post_synchronous_batch(
        data=BatchPostData(
            batchItems=[
                BatchItem(query="/search/lodz.json?limit=10&idxSet=POI,PAD,Str,Xstr,Geo,Addr"),
                BatchItem(query="/search/wroclaw.json?limit=10&idxSet=POI,PAD,Str,Xstr,Geo,Addr"),
                BatchItem(query="/search/berlin.json?limit=10&idxSet=POI,PAD,Str,Xstr,Geo,Addr"),
            ],
        ),
    )

    assert response
    assert isinstance(response, BatchResponse)


async def test_post_asynchronous_batch_submission(batch_search_api: BatchSearchApi, aresponses: ResponsesMockServer) -> None:
    """Test the post_asynchronous_batch_submission method."""
    aresponses.add(
        response=aresponses.Response(
            status=202,
            headers={"Location": "check-this-out"},
        ),
    )

    response = await batch_search_api.post_asynchronous_batch_submission(
        data=BatchPostData(
            batchItems=[
                BatchItem(query="/poiSearch/rembrandt museum.json"),
                BatchItem(query='/geometrySearch/parking.json?geometryList=[{"type":"CIRCLE","position":"51.5123443,-0.0909851","radius":1000}]'),
                BatchItem(
                    query="/geometrySearch/pizza.json",
                    post=[Geometry(type="CIRCLE", position="51.5123443,-0.0909851", radius=1000)],
                ),
                BatchItem(
                    query="/searchAlongRoute/restaurant.json?maxDetourTime=300",
                    post=[
                        Route(
                            points=[
                                LatLon(lat=37.7524152, lon=-122.4357604),
                                LatLon(lat=37.7066047, lon=-122.4330139),
                                LatLon(lat=37.7120598, lon=-122.3643493),
                                LatLon(lat=37.7535056, lon=-122.3739624),
                            ],
                        ),
                    ],
                ),
                BatchItem(query="/reverseGeocode/crossStreet/52.4829893,4.9247074.json"),
                BatchItem(query="/search/lodz.json?limit=10&idxSet=POI,PAD,Str,Xstr,Geo,Addr&maxFuzzyLevel=2"),
            ],
        ),
    )

    assert response == "check-this-out"
    assert isinstance(response, str)


@pytest.mark.usefixtures("json_response")
@pytest.mark.parametrize("json_response", ["places/batch_search/get_asynchronous_batch_download.json"], indirect=True)
async def test_deserialization_get_asynchronous_batch_download(batch_search_api: BatchSearchApi) -> None:
    """Test the get_asynchronous_batch_download method."""
    response = await batch_search_api.get_asynchronous_batch_download(
        batch_id="45e0909c-625a-4822-a060-8f7f88498c0e",
        params=AsynchronousBatchDownloadParams(
            waitTimeSeconds=10,
        ),
    )

    assert response
    assert isinstance(response, BatchResponse)



"""Test the models."""

from dataclasses import dataclass
from enum import StrEnum

from tomtom_apis.api import BaseParams


class Choice(StrEnum):
    """Test enum."""

    NO = "No"
    YES = "Yes"


@dataclass(kw_only=True)
class ParamsTest(BaseParams):
    """Test params."""

    field1: int | None = None
    field2: str | None = None
    field3: bool | None = None
    field4: float | None = None
    field5: list[str] | None = None  # Adding a list of strings
    field6: list[int] | None = None  # Adding a list of integers
    field7: Choice | None = None


def test_to_dict_all_fields_set() -> None:
    """Test to_dict method when all fields are set."""
    params = ParamsTest(field1=123, field2="test", field3=True, field4=45.67, field5=["a", "b", "c"], field6=[1, 2, 3], field7=Choice.YES)
    expected = {
        "field1": "123",
        "field2": "test",
        "field3": "true",
        "field4": "45.67",
        "field5": "a,b,c",  # List of strings converted to comma-separated string
        "field6": "1,2,3",  # List of integers converted to comma-separated string
        "field7": "Yes",
    }
    assert params.to_dict() == expected


def test_to_dict_some_fields_none() -> None:
    """Test to_dict method when some fields are None."""
    params = ParamsTest(field1=123, field2=None, field3=False, field4=None, field5=["x", "y"], field6=None, field7=None)
    expected = {"field1": "123", "field3": "false", "field5": "x,y"}
    assert params.to_dict() == expected


def test_to_dict_all_fields_none() -> None:
    """Test to_dict method when all fields are None."""
    params = ParamsTest(field1=None, field2=None, field3=None, field4=None, field5=None, field6=None, field7=None)
    expected: dict = {}
    assert params.to_dict() == expected


def test_to_dict_mixed_types_with_none() -> None:
    """Test to_dict method when some fields are None and others have different types."""
    params = ParamsTest(field1=None, field2="example", field3=None, field4=99.99, field5=["item1"], field6=[10, 20], field7=Choice.NO)
    expected = {"field2": "example", "field4": "99.99", "field5": "item1", "field6": "10,20", "field7": "No"}
    assert params.to_dict() == expected


def test_to_dict_empty_list() -> None:
    """Test to_dict method when a field is an empty list."""
    params = ParamsTest(field1=456, field2="test2", field3=False, field4=12.34, field5=[], field6=[], field7=Choice.NO)
    expected = {"field1": "456", "field2": "test2", "field3": "false", "field4": "12.34", "field7": "No"}
    assert params.to_dict() == expected


def test_to_dict_single_item_list() -> None:
    """Test to_dict method when a field is a list with one item."""
    params = ParamsTest(field1=789, field2="test3", field3=True, field4=56.78, field5=["single"], field6=[42])
    expected = {"field1": "789", "field2": "test3", "field3": "true", "field4": "56.78", "field5": "single", "field6": "42"}
    assert params.to_dict() == expected
